SystemOrganization addCategory: #'RPackage-SystemIntegration'!

BasicClassOrganizer subclass: #ClassOrganizer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Classes'!

!ClassOrganizer commentStamp: 'NS 4/6/2004 16:13' prior: 0!
I represent method categorization information for classes.  The handling of class comments has gone through a tortuous evolution.   Grandfathered class comments (before late aug 98) have no time stamps, and historically, fileouts of class comments always substituted the timestamp reflecting the author and date/time at the moment of fileout; and historically any timestamps in a filed out class comment were dropped on the floor, with the author & time prevailing at the moment of filein being substituted.   Such grandfathered comments now go out on fileouts with '<historical>' timestamp; class comments created after the 8/98 changes will have their correct timestamps preserved, though there is not yet a decent ui for reading those stamps other than filing out and looking at the file; nor is there yet any ui for browsing and recovering past versions of such comments.  Everything in good time!!!

!ClassOrganizer methodsFor: '*rpackage-systemintegration' stamp: 'EstebanLorenzano 8/3/2012 15:28'!
silentlyRenameCategory: oldCatString toBe: newCatString
	| oldCat newCat oldElementsBefore oldElementsAfter index |
	oldCat := oldCatString asSymbol.
	newCat := newCatString asSymbol.
	oldElementsBefore := self listAtCategoryNamed: oldCat.
	(index := categoryArray indexOf: oldCat) = 0
		ifTrue: [^ self].	"old name not found, so no action"
	(categoryArray indexOf: newCat) > 0
		ifFalse: [
			categoryArray := categoryArray copy.  "need to change identity so smart list update will notice the change"
			categoryArray at: index put: newCat
			]
		ifTrue: [
			"if the category already exist, we move all elements inside and remove the old category"
			SystemAnnouncer uniqueInstance suspendAllWhile: [
				oldElementsBefore do: [:anElement |
					self classify: anElement under: newCat
					].
			self removeCategory:  oldCat   				
				].
			].! !

SystemAnnouncement subclass: #ClassRepackaged
	instanceVariableNames: 'classRepackaged newPackage oldPackage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage-SystemIntegration'!

!ClassRepackaged class methodsFor: 'instance creation' stamp: 'GuillermoPolito 8/17/2012 17:20'!
classRepackaged: aClass oldPackage: oldPackage newPackage: newPackage
	^self new
			classRepackaged: aClass;
			oldPackage: oldPackage;
			newPackage: newPackage;
			yourself.! !

!ClassRepackaged methodsFor: 'accessing' stamp: 'GuillermoPolito 8/17/2012 17:18'!
classRepackaged
	
	^ classRepackaged! !

!ClassRepackaged methodsFor: 'accessing' stamp: 'GuillermoPolito 8/17/2012 17:18'!
classRepackaged: anObject
	
	classRepackaged := anObject! !

!ClassRepackaged methodsFor: 'accessing' stamp: 'GuillermoPolito 8/17/2012 17:18'!
newPackage
	
	^ newPackage! !

!ClassRepackaged methodsFor: 'accessing' stamp: 'GuillermoPolito 8/17/2012 17:18'!
newPackage: anObject
	
	newPackage := anObject! !

!ClassRepackaged methodsFor: 'accessing' stamp: 'GuillermoPolito 8/17/2012 17:18'!
oldPackage
	
	^ oldPackage! !

!ClassRepackaged methodsFor: 'accessing' stamp: 'GuillermoPolito 8/17/2012 17:18'!
oldPackage: anObject
	
	oldPackage := anObject! !

SystemAnnouncement subclass: #MethodRepackaged
	instanceVariableNames: 'methodRepackaged newPackage oldPackage'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage-SystemIntegration'!

!MethodRepackaged class methodsFor: 'instance creation' stamp: 'GuillermoPolito 8/17/2012 17:20'!
methodRepackaged: aMethod oldPackage: oldPackage newPackage: newPackage
	^self new
			methodRepackaged: aMethod;
			oldPackage: oldPackage;
			newPackage: newPackage;
			yourself.! !

!MethodRepackaged methodsFor: 'accessing' stamp: 'GuillermoPolito 8/17/2012 17:18'!
methodRepackaged
	
	^ methodRepackaged! !

!MethodRepackaged methodsFor: 'accessing' stamp: 'GuillermoPolito 8/17/2012 17:18'!
methodRepackaged: anObject
	
	methodRepackaged := anObject! !

!MethodRepackaged methodsFor: 'accessing' stamp: 'GuillermoPolito 8/17/2012 17:18'!
newPackage
	
	^ newPackage! !

!MethodRepackaged methodsFor: 'accessing' stamp: 'GuillermoPolito 8/17/2012 17:18'!
newPackage: anObject
	
	newPackage := anObject! !

!MethodRepackaged methodsFor: 'accessing' stamp: 'GuillermoPolito 8/17/2012 17:18'!
oldPackage
	
	^ oldPackage! !

!MethodRepackaged methodsFor: 'accessing' stamp: 'GuillermoPolito 8/17/2012 17:18'!
oldPackage: anObject
	
	oldPackage := anObject! !

Model subclass: #MCPackageManager
	instanceVariableNames: 'package modified'
	classVariableNames: 'PrivateAnnouncer'
	poolDictionaries: ''
	category: 'Monticello-Versioning'!

MCPackageManager class
	instanceVariableNames: 'registry'!

!MCPackageManager commentStamp: 'StephaneDucasse 4/29/2011 20:42' prior: 0!
MCPackageManager is a kind of package wrapper knowing if the package is dirty or not.
The class side manages registered package managers.

Instance Variables:
	package	<MCPackage>
	modified	<Boolean>

Class Instance Variables:
	registry	<Dictionary>!

MCPackageManager class
	instanceVariableNames: 'registry'!

!MCPackageManager class methodsFor: '*rpackage-systemintegration' stamp: 'CamilleTeruel 7/29/2012 18:45'!
unregisterForNotifications
	SystemAnnouncer uniqueInstance unsubscribe: self! !

RPackageTestCase subclass: #RPackageMCSynchronisationTest
	instanceVariableNames: 'emptyOrganizer oldAnnouncer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage-SystemIntegration'!

!RPackageMCSynchronisationTest commentStamp: '<historical>' prior: 0!
I had some problems using the 'ensure:' method. There was some conflicts with the organizer used for the test when trying to clean the classes, categories and packages generated for the test. 
There is something I don't get with this method. Therefore I choosed to clean by default all possible generated classes, categories and packages after each test using the tearDown method.

For now, all that was in the 'ensure:' methods has been commented.
Look for example at:
    testAddMethodInClassicCategoryAddMethodToTheParentPackageOfItsClass
and try to uncomment it!

!RPackageMCSynchronisationTest methodsFor: 'utilities' stamp: 'GuillermoPolito 5/29/2012 12:31'!
addXCategory
	Smalltalk globals organization addCategory: 'XXXXX'.! !

!RPackageMCSynchronisationTest methodsFor: 'utilities' stamp: 'StephaneDucasse 4/29/2011 21:44'!
addXYCategory

	self addXCategory.
	self addYCategory.
			! !

!RPackageMCSynchronisationTest methodsFor: 'utilities' stamp: 'StephaneDucasse 4/29/2011 21:50'!
addXYZCategory

	self addXCategory.
	self addYCategory.
	self addZCategory.
			! !

!RPackageMCSynchronisationTest methodsFor: 'utilities' stamp: 'StephaneDucasse 4/24/2011 23:26'!
addYCategory

	Smalltalk globals organization addCategory: 'YYYYY'.
			! !

!RPackageMCSynchronisationTest methodsFor: 'utilities' stamp: 'StephaneDucasse 4/29/2011 21:44'!
addZCategory

	Smalltalk globals organization addCategory: 'ZZZZZ'.
			! !

!RPackageMCSynchronisationTest methodsFor: 'utilities' stamp: 'StephaneDucasse 6/2/2012 19:40'!
allManagers

	^ MCWorkingCopy allManagers! !

!RPackageMCSynchronisationTest methodsFor: 'setup' stamp: 'StephaneDucasse 6/2/2012 19:45'!
cleanClassesPackagesAndCategories

		|mCPackage|
		Smalltalk removeClassNamed: 'NewClass'.
		Smalltalk removeClassNamed: 'RPackageNewStubClass'.
		Smalltalk removeClassNamed: 'RPackageOldStubClass'.
		Smalltalk removeClassNamed: 'Foo'.
		Smalltalk removeClassNamed: 'FooOther'.
		Smalltalk removeClassNamed: 'NewTrait'.
		Smalltalk removeClassNamed: 'ClassInYPackage'.
		Smalltalk removeClassNamed: 'ClassInZPackage'.
		Smalltalk organization removeCategory: 'Zork'.
		Smalltalk organization removeCategory: 'XXXXX'.
		Smalltalk organization removeCategory: 'XXXX'.
		Smalltalk organization removeCategory: 'YYYYY'.
		Smalltalk organization removeCategory: 'ZZZZZ'.
		Smalltalk organization removeCategory: 'FooPackage-Core'.
		Smalltalk organization removeCategory: 'FooPackage-Other'.
		Smalltalk organization removeCategory: 'FooPackage'.
		Smalltalk organization removeCategory: 'OriginalCategory'.
		Smalltalk organization removeCategory: 'NewCategoryName'.
		Smalltalk organization removeCategory: 'Y'.
		mCPackage := (self allManagers detect: [:each | each packageName = 'OriginalCategory'] ifNone: [nil]) .
		mCPackage ifNotNil: [mCPackage unregister].
		mCPackage := (self allManagers detect: [:each | each packageName = 'XXXXX'] ifNone: [nil]) .
		mCPackage ifNotNil: [mCPackage unregister].
		mCPackage := (self allManagers detect: [:each | each packageName = 'XXXX'] ifNone: [nil]) .
		mCPackage ifNotNil: [mCPackage unregister].
		mCPackage := (self allManagers detect: [:each | each packageName = 'YYYYY'] ifNone: [nil]) .
		mCPackage ifNotNil: [mCPackage unregister].
		mCPackage := (self allManagers detect: [:each | each packageName = 'YYYY'] ifNone: [nil]) .
		mCPackage ifNotNil: [mCPackage unregister].
		mCPackage := (self allManagers detect: [:each | each packageName = 'Yyyyy'] ifNone: [nil]) .
		mCPackage ifNotNil: [mCPackage unregister].	
		mCPackage := (self allManagers detect: [:each | each packageName = 'YyYyY'] ifNone: [nil]) .
		mCPackage ifNotNil: [mCPackage unregister].
		mCPackage := (self allManagers detect: [:each | each packageName = 'Y'] ifNone: [nil]) .
		mCPackage ifNotNil: [mCPackage unregister].
		mCPackage := (self allManagers detect: [:each | each packageName = 'ZZZZZ'] ifNone: [nil]) .
		mCPackage ifNotNil: [mCPackage unregister].	
		mCPackage := (self allManagers detect: [:each | each packageName = 'Zzzzz'] ifNone: [nil]) .
		mCPackage ifNotNil: [mCPackage unregister].
		mCPackage := (self allManagers detect: [:each | each packageName = 'FooPackage-Core'] ifNone: [nil]) .
		mCPackage ifNotNil: [mCPackage unregister].
		mCPackage := (self allManagers detect: [:each | each packageName = 'FooPackage-Other'] ifNone: [nil]) .
		mCPackage ifNotNil: [mCPackage unregister].
		mCPackage := (self allManagers detect: [:each | each packageName = 'FooPackage'] ifNone: [nil]) .
		mCPackage ifNotNil: [mCPackage unregister].
		mCPackage := (self allManagers detect: [:each | each packageName = 'Zork'] ifNone: [nil]) .
		mCPackage ifNotNil: [mCPackage unregister].
			! !

!RPackageMCSynchronisationTest methodsFor: 'private' stamp: 'cyrille.delaunay 11/25/2010 13:17'!
createMethodNamed: methodName inClass: aClass inCategory: aCategoryName 

	^ aClass compile: (methodName, ' ^nil') classified: aCategoryName.! !

!RPackageMCSynchronisationTest methodsFor: 'private' stamp: 'cyrille.delaunay 11/25/2010 13:17'!
createMethodNamed: methodName inClassSideOfClass: aClass inCategory: aCategoryName 

	^ aClass classSide compile: (methodName, ' ^nil') classified: aCategoryName.! !

!RPackageMCSynchronisationTest methodsFor: 'private' stamp: 'StephaneDucasse 7/29/2010 23:46'!
createNewClassNamed: aName
	
	| cls |
	cls := Object subclass: aName
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: 'RPackageTest'.
	^ cls! !

!RPackageMCSynchronisationTest methodsFor: 'private' stamp: 'cyrille.delaunay 11/22/2010 10:56'!
createNewClassNamed: aName inCategory: aCategoryName
	
	| cls |
	cls := Object subclass: aName asSymbol
		instanceVariableNames: ''
		classVariableNames: ''
		poolDictionaries: ''
		category: aCategoryName.
	^ cls! !

!RPackageMCSynchronisationTest methodsFor: 'private' stamp: 'cyrille.delaunay 11/26/2010 15:31'!
createNewTraitNamed: aName inCategory: aCategoryName
	
	| trait |
	trait := Trait named: aName asSymbol
			uses: {}
			category: aCategoryName.
	^ trait! !

!RPackageMCSynchronisationTest methodsFor: 'accessing' stamp: 'RPackage 1/20/2011 13:17'!
emptyOrganizer 

	^ emptyOrganizer! !

!RPackageMCSynchronisationTest methodsFor: 'announcer handling' stamp: 'EstebanLorenzano 7/27/2012 17:57'!
initializeAnnouncers
	super initializeAnnouncers.
	
	oldAnnouncer := MCWorkingCopy announcer.
	MCWorkingCopy announcer: announcerForTest.! !

!RPackageMCSynchronisationTest methodsFor: 'announcer handling' stamp: 'EstebanLorenzano 7/27/2012 17:50'!
restoreAnnouncers
	super restoreAnnouncers.
	MCWorkingCopy announcer: oldAnnouncer.! !

!RPackageMCSynchronisationTest methodsFor: 'setup' stamp: 'EstebanLorenzano 7/27/2012 20:24'!
setUp 
	super setUp.
	
	emptyOrganizer := self organizer debuggingName: 'MCSynchronisation Package Organizer'; yourself.
	emptyOrganizer registerPackage: (self packageClass named: 'as yet unclassified').

	Author fullName 
		ifNil: [ Author fullName: 'Tester' ].! !

!RPackageMCSynchronisationTest methodsFor: 'setup' stamp: 'EstebanLorenzano 7/27/2012 18:01'!
tearDown
	MCWorkingCopy removeDependent: self emptyOrganizer.
	self cleanClassesPackagesAndCategories.
	super tearDown.! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on categories' stamp: 'GuillermoPolito 5/26/2012 22:48'!
testAddCategoryCreatesPackage
	"test that when we add a category that does not yet correspond to an existing package, a new package is created"
	"self debug: #testAddCategoryCreatesPackage"
	
	self addXCategory.
	self assert: (self organizer includesPackageNamed: #XXXXX)! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on categories' stamp: 'GuillermoPolito 5/26/2012 22:48'!
testAddCategoryIsAlreadyAPackageDoesNotCreatePackage
	"test that when we add a category corresponding to an existing package, we do not create a second package"
	"self debug: #testAddCategoryIsAlreadyAPackageDoesNotCreatePackage"
	
	|tmpPackage|
	self addXCategory.
	tmpPackage := self organizer packageNamed: #XXXXX.
	self addXCategory.
	self assert: tmpPackage =  (self organizer packageNamed: #XXXXX)
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'GuillermoPolito 5/26/2012 22:48'!
testAddClassAddItIntoPackage
	"self debug: #testAddCategoryCreatesPackage"

	|tmpPackage class|
	self addXCategory.
	self assert: (self organizer includesPackageNamed: #XXXXX).
	tmpPackage := self organizer packageNamed: #XXXXX.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self assert: (tmpPackage definesClass: class). 
	self assert: (class package = tmpPackage)! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'GuillermoPolito 5/26/2012 22:48'!
testAddClassInNotCreatedPackage
	"test that when we add a class in an existing category, it does not create a new package"
	"self debug: #testAddClassInNotCreatedPackage"
	
	|tmpPackage class|
	self addXCategory.
	self assert: (self organizer includesPackageNamed: #XXXXX).
	tmpPackage := self organizer packageNamed: #XXXXX.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self assert: (self organizer packageNamed: #XXXXX) = tmpPackage. 
	self assert: (class package = tmpPackage)! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'GuillermoPolito 5/26/2012 22:49'!
testAddClassUpdateTheOrganizer
	"test that when we add a class, the organizer 'classPackageMapping' dictionary is updated, so that the class is linked to its package and so that we can access its parent package"
	"self debug: #testAddClassUpdateTheOrganizer"
	|tmpPackage class|
	self addXCategory.
	self assert: (self organizer includesPackageNamed: #XXXXX).
	tmpPackage := self organizer packageNamed: #XXXXX.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self assert: (self organizer packageOf: class) = tmpPackage. 
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on MCPackages' stamp: 'GuillermoPolito 5/26/2012 22:49'!
testAddMCPackageCreatesAPackage
	"test that when we create a MCPackage, a corresponding package is created"
	
	MCWorkingCopy forPackage: (MCPackage new name: #Zork).
	self assert: (self organizer includesPackageNamed: #Zork).
		
					  ! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on MCPackages' stamp: 'GuillermoPolito 5/26/2012 22:49'!
testAddMCPackageForCategoryAlreadyExistingDoesNotCreateAPackage
	"test that when we create a MCPackage and that a category of this name already exists, no package is created"
	"self debug: #testAddMCPackageForCategoryAlreadyExistingDoesNotCreateAPackage"
	"self run: #testAddMCPackageForCategoryAlreadyExistingDoesNotCreateAPackage"	
	
	|tmpPackage|
	Smalltalk globals organization addCategory: 'Zork'.
	tmpPackage := self organizer packageNamed: #Zork.
	MCWorkingCopy forPackage: (MCPackage new name: #Zork).
	self assert: tmpPackage = (self organizer packageNamed: #Zork).
		
		
					  ! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 5/26/2012 22:51'!
testAddMethodByUsingATraitDoesNotAddTheMethodToThePackage
	"test that when a method is added to a class bu using a trait, this method is not imported in the parent package of the class"
	"self debug: #testAddMethodByUsingATraitDoesNotAddTheMethodToThePackage"
	
	|XPackage YPackage class trait|
	self addXCategory.
	self addYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage := self organizer packageNamed: #YYYYY.
	trait := self createNewTraitNamed: 'NewTrait' inCategory: 'YYYYY'.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass:  trait inCategory: 'trait category'.
	
	self assert: (XPackage methods isEmpty ).
	self deny: (YPackage methods isEmpty ).
	class addToComposition: trait.
	self assert: (YPackage includesDefinedSelector: #stubMethod ofClass: trait).
	self assert: (XPackage methods isEmpty).
		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 5/26/2012 22:52'!
testAddMethodInClassicCategoryAddMethodToTheParentPackageOfItsClass
	"test that when we add a method to a  class in a classic category (*not beginning with *), this method is added to the parent package of the class"
	
	|tmpPackage class|
	self addXCategory.
	self assert: (self organizer includesPackageNamed: #'XXXXX').
	tmpPackage := self organizer packageNamed: #XXXXX.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass:  class inCategory: 'classic category'.
	self assert: (class package includesDefinedSelector: #stubMethod ofClass: class).
	self assert: ((class >> #stubMethod) packageFromOrganizer: self organizer ) = class package.! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 5/26/2012 22:52'!
testAddMethodInClassicCategoryAddMethodToTheParentPackageOfItsTrait
	"test that when we add a method to a  trait in a classic category (*not beginning with *), this method is added to the parent package of the class"
	
	|tmpPackage trait|
	self addXCategory.
	self assert: (self organizer includesPackageNamed: #'XXXXX').
	tmpPackage := self organizer packageNamed: #XXXXX.
	trait := self createNewTraitNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass:  trait inCategory: 'classic category'.
	self assert: (tmpPackage includesClass: trait).
	self assert: (trait package includesDefinedSelector: #stubMethod ofClass: trait).
	self assert: ((trait >> #stubMethod) packageFromOrganizer: self organizer ) = trait package! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 5/26/2012 22:53'!
testAddMethodInExtensionCategoryAddMethodToTheExtendingPackage
	"test that when we add a method to a  class in an extension category ( beginning with *), this method is added to the corresponding extending package"
	
	| class firstPackage secondPackage|
	self addXYCategory.		
	firstPackage := self organizer packageNamed: #XXXXX.
	secondPackage := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self assert: (class package = firstPackage).
	self createMethodNamed: 'stubMethod' inClass: class inCategory: '*YYYYY'.
	self assert: (secondPackage includesExtensionSelector: #stubMethod ofClass: class).
	self deny: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
	self assert: ((class >> #stubMethod) packageFromOrganizer: self organizer) = secondPackage! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 5/26/2012 22:54'!
testAddMethodInExtensionCategoryEnlargingPackageNameAddMethodToTheExtendingPackage
	"test that when we add a method  in an extension category ( beginning with*) that enlarge a package name (for example *mondrian-accessing for Mondrian), this method is added to the corresponding extending package"
	
	| class XPackage YPackage|
	self addXYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self assert: class package = XPackage.
	self createMethodNamed: #newMethod inClass:  class inCategory: '*YYYYY-subcategory'.
	self assert: (YPackage includesExtensionSelector: #newMethod ofClass: class).
	self deny: (XPackage includesDefinedSelector: #newMethod ofClass: class).
	self assert: ((class >> #newMethod) packageFromOrganizer: self organizer ) = YPackage! !

!RPackageMCSynchronisationTest methodsFor: 'expectedfailures' stamp: 'GuillermoPolito 5/26/2012 22:55'!
testAddMethodInExtensionCategoryNotExistingCreateANewPackage
	"test that when we add a method  in an extension category ( beginning with *)that does not refer to an existing categorya new package with the name of this category is added, and that the method is added to this new package"
	| class firstPackage secondPackage|
	self flag: #stef. "this is strange since the setup does not use withOrganizer:do: and now we use it. There are some inconsistencies"
	"not sure this is the behavior expected. Maybe a new package should be created and should contain the method. Actually, this is the reason the test fail :)."
	
	Smalltalk globals organization addCategory: 'XXXXX'.
	Smalltalk globals organization addCategory: 'YYYYY'.
	firstPackage := self organizer packageNamed: #XXXXX.
	secondPackage := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: #newMethod inClass:  class inCategory: '*YYYY'.
	self deny: (secondPackage includesExtensionSelector: #newMethod ofClass: class).
	self deny: (class package includesDefinedSelector: #newMethod ofClass: class).
	self assert: ((self organizer packageNamed: #YYYY) = nil) not.
	self assert: ((class >> #newMethod) packageFromOrganizer: self organizer ) = (self organizer packageNamed: #YYYY).! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 5/26/2012 22:56'!
testAddMethodInExtensionCategoryNotRespectingCaseAddMethodToTheExtendingPackage
	"test that when we add a method  in an extension category ( beginning with *)thae does not match the case of the corresponding package (for example *packagea for PackageA), this method is added to the corresponding extending package"
	

	| class firstPackage secondPackage|
	self addXYCategory.
	firstPackage := self organizer packageNamed: #XXXXX.
	secondPackage := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass:  class inCategory: '*yyYyY'.
	self assert: (secondPackage includesExtensionSelector: #stubMethod ofClass: class).
	self deny: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
	self assert: ((class >> #stubMethod) packageFromOrganizer: self organizer ) = secondPackage.! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on traits' stamp: 'GuillermoPolito 5/26/2012 22:58'!
testAddTraitAddItIntoPackage
	"self debug: #testAddCategoryCreatesPackage"

	|tmpPackage trait|
	self addXCategory.
	self assert: (self organizer includesPackageNamed: 'XXXXX').
	tmpPackage := self organizer packageNamed: #XXXXX.
	trait := self createNewTraitNamed: 'NewTrait' inCategory: 'XXXXX'.
	self assert: (tmpPackage definesClass: trait).! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on traits' stamp: 'GuillermoPolito 5/26/2012 22:59'!
testAddTraitInNotCreatedPackage
	"test that when we add a trait in an existing category, it does not create a new package"
	
	|tmpPackage trait|
	self addXCategory.
	self assert: (self organizer includesPackageNamed: 'XXXXX').
	tmpPackage := self organizer packageNamed: #XXXXX.
	trait := self createNewClassNamed: 'NewTrait' inCategory: 'XXXXX'.
	self assert: (self organizer packageNamed: #XXXXX) = tmpPackage. 
	self assert: (trait package = tmpPackage)! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on traits' stamp: 'GuillermoPolito 5/26/2012 23:01'!
testAddTraitUpdateTheOrganizer
	"test that when we add a trait, the organizer 'classPackageMapping' dictionary is updated, so that the class is linked to its package and so that we can access its parent package"
	
	|tmpPackage trait|
	self addXCategory.
	self assert: (self organizer includesPackageNamed: 'XXXXX').
	tmpPackage := self organizer packageNamed: #XXXXX.
	trait := self createNewTraitNamed: 'NewTrait' inCategory: 'XXXXX'.
	self assert: (self organizer packageOf: trait) = tmpPackage.! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'StephaneDucasse 6/3/2012 22:50'!
testBasicImportSystemClass
	"self debug: #testBasicImportSystemClass"
	| fooClass fooPackage |
	fooClass :=  Object subclass: #Foo
					instanceVariableNames: ''
					classVariableNames: ''
					poolDictionaries: ''
					category: 'FooPackage'.
	fooClass compileSilently: 'foo ^ 1' classified: 'foo'.
	fooClass compileSilently: 'fooBar ^ 2' classified: 'foo'.
	fooClass compileSilently: 'fooFooBar ^ 3' classified: 'foo'.
	fooClass class compileSilently: 'fooClassSide ^ 1' classified: 'foo'.
	fooClass compileSilently: 'zork ^ 4' classified: '*zork'.
	fooPackage := self packageClass named: #FooPackage.
	fooPackage importCategory.
	self assert: (fooPackage includesDefinedSelector: #foo ofClass: fooClass).
	self assert: (fooPackage includesDefinedSelector: #fooBar ofClass: fooClass).
	self assert: (fooPackage includesDefinedSelector: #fooFooBar ofClass: fooClass).
	self deny: (fooPackage includesDefinedSelector: #zork ofClass: fooClass).
	self assert: (fooPackage includesDefinedSelector: #fooClassSide ofClass: fooClass class).! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'GuillermoPolito 5/26/2012 23:09'!
testCategoryMatching
	
	self assert: (self organizer category: 'Compression-Archives' matches: 'Compression').
	self deny: (self organizer category: 'Compression' matches: 'Compression-Archives').
	self assert: (self organizer category: 'Compression' matches: 'Compression').
	self assert: (self organizer category: 'Compression-Arh' matches: 'Compression').
	self deny: (self organizer category: 'XXXX' matches: 'Compression-Archives').! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'StephaneDucasse 6/3/2012 22:50'!
testImportSystallManagershSubCategoryAndAnotherCategory
	"self debug: #testImportSystemClassWithSubCategoryAndAnotherCategory"
	"indeed the user may have created a class in the category Foo-Core, and after created
	a package only named Foo. The classes in category Foo-Core should be added to Foo. But
	some other packages and categories starting with Foo may exist."
	| fooClass fooPackage fooClass2|
	fooClass := self createNewClassNamed: 'Foo' inCategory: 'FooPackage-Core'.
	fooClass compileSilently: 'foo ^ 1' classified: 'foo'.
	fooClass compileSilently: 'fooBar ^ 2' classified: 'foo'.
	fooClass compileSilently: 'fooFooBar ^ 3' classified: 'foo'.
	fooClass class compileSilently: 'fooClassSide ^ 1' classified: 'foo'.
	fooClass compileSilently: 'zork ^ 4' classified: '*zork'.

	fooClass2 := self createNewClassNamed: 'FooOther' inCategory: 'FooPackage-Other'.
	fooClass2 compileSilently: 'fooOther ^ 1' classified: 'what s up'.
	fooClass2 compileSilently: 'fooBarOther ^ 2' classified: 'dr'.
	fooClass2 class compileSilently: 'fooOtherClassSide ^ 1' classified: 'lovely carots'.
	fooClass2 compileSilently: 'zorkZork ^ 4' classified: '*zorkzork'.
	
	fooPackage := self packageClass named: #FooPackage.
	fooPackage importCategory.
	self assert: (fooPackage definedClasses size = 2).
	
	self assert: (fooPackage includesDefinedSelector: #foo ofClass: fooClass).
	self assert: (fooPackage includesDefinedSelector: #fooBar ofClass: fooClass).
	self assert: (fooPackage includesDefinedSelector: #fooFooBar ofClass: fooClass).
	self deny: (fooPackage includesDefinedSelector: #zork ofClass: fooClass).
	self assert: (fooPackage includesDefinedSelector: #fooClassSide ofClass: fooClass class).
	
	self assert: (fooPackage includesDefinedSelector: #fooOther ofClass: fooClass2).
	self assert: (fooPackage includesDefinedSelector: #fooBarOther ofClass: fooClass2).
	self deny: (fooPackage includesDefinedSelector: #zorkZork ofClass: fooClass2).
	self assert: (fooPackage includesDefinedSelector: #fooOtherClassSide ofClass: fooClass2 class).! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'StephaneDucasse 6/3/2012 22:50'!
testImportSystemClassWithSubCategory
	"self debug: #testImportSystemClassWithSubCategory"
	"indeed the user may have created a class in the category Foo-Core, and after created
	a package only named Foo. The classes in category Foo-Core should be added to Foo. But
	some other packages and categories starting with Foo may exist."
	| fooClass fooPackage |
	fooClass :=  Object subclass: #Foo
					instanceVariableNames: ''
					classVariableNames: ''
					poolDictionaries: ''
					category: 'FooPackage-Core'.
	fooClass compileSilently: 'foo ^ 1' classified: 'foo'.
	fooClass compileSilently: 'fooBar ^ 2' classified: 'foo'.
	fooClass compileSilently: 'fooFooBar ^ 3' classified: 'foo'.
	fooClass class compileSilently: 'fooClassSide ^ 1' classified: 'foo'.
	fooClass compileSilently: 'zork ^ 4' classified: '*zork'.
	fooPackage := self packageClass named: #FooPackage.
	fooPackage importCategory.
	self assert: (fooPackage includesDefinedSelector: #foo ofClass: fooClass).
	self assert: (fooPackage includesDefinedSelector: #fooBar ofClass: fooClass).
	self assert: (fooPackage includesDefinedSelector: #fooFooBar ofClass: fooClass).
	self deny: (fooPackage includesDefinedSelector: #zork ofClass: fooClass).
	self assert: (fooPackage includesDefinedSelector: #fooClassSide ofClass: fooClass class).! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'StephaneDucasse 6/3/2012 22:50'!
testImportSystemClassWithSubCategoryAndAnotherCategory
	"self debug: #testImportSystemClassWithSubCategoryAndAnotherCategory"
	"indeed the user may have created a class in the category Foo-Core, and after created
	a package only named Foo. The classes in category Foo-Core should be added to Foo. But
	some other packages and categories starting with Foo may exist."
	
	| fooClass fooPackage fooClass2|
	fooClass :=  Object subclass: #Foo
					instanceVariableNames: ''
					classVariableNames: ''
					poolDictionaries: ''
					category: 'FooPackage-Core'.
	fooClass compileSilently: 'foo ^ 1' classified: 'foo'.
	fooClass compileSilently: 'fooBar ^ 2' classified: 'foo'.
	fooClass compileSilently: 'fooFooBar ^ 3' classified: 'foo'.
	fooClass class compileSilently: 'fooClassSide ^ 1' classified: 'foo'.
	fooClass compileSilently: 'zork ^ 4' classified: '*zork'.
	
	fooClass2 :=  Object subclass: #FooOther
					instanceVariableNames: ''
					classVariableNames: ''
					poolDictionaries: ''
					category: 'FooPackage-Other'.
	fooClass2 compileSilently: 'fooOther ^ 1' classified: 'what s up'.
	fooClass2 compileSilently: 'fooBarOther ^ 2' classified: 'dr'.
	fooClass2 class compileSilently: 'fooOtherClassSide ^ 1' classified: 'lovely carots'.
	fooClass2 compileSilently: 'zorkZork ^ 4' classified: '*zorkzork'.
	
	fooPackage := self packageClass named: #FooPackage.
	fooPackage importCategory.
	self assert: (fooPackage definedClasses size = 2).
	self assert: (fooPackage includesDefinedSelector: #foo ofClass: fooClass).
	self assert: (fooPackage includesDefinedSelector: #fooBar ofClass: fooClass).
	self assert: (fooPackage includesDefinedSelector: #fooFooBar ofClass: fooClass).
	self deny: (fooPackage includesDefinedSelector: #zork ofClass: fooClass).
	self assert: (fooPackage includesDefinedSelector: #fooClassSide ofClass: fooClass class).
	self assert: (fooPackage includesDefinedSelector: #fooOther ofClass: fooClass2).
	self assert: (fooPackage includesDefinedSelector: #fooBarOther ofClass: fooClass2).
	self deny: (fooPackage includesDefinedSelector: #zorkZork ofClass: fooClass2).
	self assert: (fooPackage includesDefinedSelector: #fooOtherClassSide ofClass: fooClass2 class).! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 5/26/2012 23:03'!
testModifyMethodByChangingCode
	"test that when we modify the code the code of a method, everything work well "
	
	| class firstPackage secondPackage|
	self addXCategory.
	firstPackage := self organizer  packageNamed: #XXXXX.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass:  class inCategory: 'classic category'.

	self assert: (class >> #stubMethod) category = 'classic category'. 
	self assert: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
	self deny: (firstPackage includesExtensionSelector: #stubMethod ofClass: class).
	self assert: ((class >> #stubMethod) packageFromOrganizer: self organizer ) = firstPackage.
	
	class compile: 'stubMethod ^22222222222'.
	
	"nothing should change"
	self assert: (class >> #stubMethod) category = 'classic category'. 
	self assert: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
	self deny: (firstPackage includesExtensionSelector: #stubMethod ofClass: class).
	self assert: ((class >> #stubMethod) packageFromOrganizer: self organizer ) = firstPackage.! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 5/26/2012 23:04'!
testModifyMethodByMovingFromClassicCategoryToClassicCategoryDoesNothing
	"test that when we move a method from a classic category (not begining with *) to another classic category , nothing is modified"
	
	| class firstPackage secondPackage|
	self addXCategory.
	firstPackage := self organizer packageNamed: #XXXXX.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass:  class inCategory: 'classic category'.

	self assert: (class >> #stubMethod) category = 'classic category'. 
	self assert: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
	self deny: (firstPackage includesExtensionSelector: #stubMethod ofClass: class).
	self assert: ((class >> #stubMethod) packageFromOrganizer: self organizer ) = firstPackage.
	
	class organization classify: #stubMethod under: 'new category'.
	self assert: (class >> #stubMethod) category = 'new category'. 
	self assert: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
	self deny: (firstPackage includesExtensionSelector: #stubMethod ofClass: class).
	self assert: ((class >> #stubMethod) packageFromOrganizer: self organizer ) = firstPackage! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 5/26/2012 23:05'!
testModifyMethodByMovingFromClassicCategoryToExtensionCategoryMoveItFromParentPackageToExtendingPackage
	"test that when we move a method from a classic category (not begining with *) to an extension category , the method is moved from the parent package of the class to the extending package"
	
	| class firstPackage secondPackage|
	self addXYCategory. 
	firstPackage := self organizer  packageNamed: #XXXXX.
	secondPackage := self organizer  packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass:  class inCategory: 'classic category'.

	self assert: (class >> #stubMethod) category = 'classic category'. 
	self assert: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
	self deny: (firstPackage includesExtensionSelector: #stubMethod ofClass: class).
	self assert: ((class >> #stubMethod) packageFromOrganizer: self organizer ) = firstPackage.
	
	class organization classify: #stubMethod under: '*yyyyy'.	
	self assert: (class >> #stubMethod) category = '*yyyyy'. 
	self deny: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
	self assert: (secondPackage includesExtensionSelector: #stubMethod ofClass: class).
	self assert: ((class >> #stubMethod) packageFromOrganizer: self organizer ) = secondPackage.
	
	class organization classify: #stubMethod under: '*yyyyy-suncategory'.
	"class compileSilently: (class>>#stubMethod) getSource classified: '*yyyyy-suncategory'."
	self assert: (class >> #stubMethod) category = '*yyyyy-suncategory'. 
	self assert: ((class >> #stubMethod) packageFromOrganizer: self organizer ) = secondPackage.! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 5/26/2012 23:05'!
testModifyMethodByMovingFromExtensionCategoryToClassicCategoryMoveItFromExtendingPackageToParentPackage

	"test that when we move a method from an extension category ( begining with *) to a classic category , the method is moved from the extending package to the parent package of the class"
	
	| class firstPackage secondPackage|
	self addXYCategory.
	firstPackage := self organizer  packageNamed: #XXXXX.
	secondPackage := self organizer  packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass:  class inCategory: '*yyyyy'.

	self assert: (class >> #stubMethod) category = '*yyyyy'. 
	self deny: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
	self assert: (secondPackage includesExtensionSelector: #stubMethod ofClass: class).
	self assert: ((class >> #stubMethod) packageFromOrganizer: self organizer ) = secondPackage.
	
	class organization classify: #stubMethod under: 'classic category'.
	
	self assert: (class >> #stubMethod) category = 'classic category'. 
	self assert: (firstPackage includesDefinedSelector: #stubMethod ofClass: class).
	self deny: (secondPackage includesExtensionSelector: #stubMethod ofClass: class).
	self assert: ((class >> #stubMethod) packageFromOrganizer: self organizer ) = firstPackage.! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 5/26/2012 23:07'!
testModifyMethodByMovingFromExtensionCategoryToExtensionCategoryMoveItFromExtendingPackageToSecondExtendingPackage
	"test that when we move a method from an extension category ( begining with *) to another extending package , the method is moved from the extending package to the other extending package"
	
	| class XPackage YPackage ZPackage|
	self addXYZCategory.
	XPackage := self organizer  packageNamed: #XXXXX.
	YPackage := self organizer  packageNamed: #YYYYY.
	ZPackage := self organizer  packageNamed: #ZZZZZ.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: #newMethod inClass:  class inCategory: '*yyyyy'.

	self assert: (class >> #newMethod) category = '*yyyyy'. 
	self deny: (XPackage includesDefinedSelector: #newMethod ofClass: class).
	self assert: (YPackage includesExtensionSelector: #newMethod ofClass: class).
	self assert: ((class >> #newMethod) packageFromOrganizer: self organizer ) = YPackage.
	
	class organization classify: #newMethod under: '*zzzzz'.
	self assert: (class >> #newMethod) category = '*zzzzz'. 
	self deny: (XPackage includesDefinedSelector: #newMethod ofClass: class).
	self deny: (YPackage includesExtensionSelector: #newMethod ofClass: class).
	self assert: (ZPackage includesExtensionSelector: #newMethod ofClass: class).
	self assert: ((class >> #newMethod) packageFromOrganizer: self organizer ) = ZPackage.! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 8/17/2012 17:40'!
testNoptRepackagedAnnouncementWhenMovingClassicCategories

	| ann class firstPackage secondPackage|
	
	ann := nil.
	SystemAnnouncer uniqueInstance
		on: MethodRepackaged do: [ :a | ann := a ].
	
	self addXYCategory. 
	firstPackage := self organizer  packageNamed: #XXXXX.
	secondPackage := self organizer  packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.

	self createMethodNamed: 'stubMethod' inClass:  class inCategory: 'classic'.
	
	self assert: ann isNil.

	class organization classify: #stubMethod under: 'another classic one'.

	self assert: ann isNil.! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 8/17/2012 17:41'!
testNotRepackagedAnnouncementWhenModifyMethodByMovingInSameExtensionCategory
	
	| ann class firstPackage secondPackage|
	
	ann := nil.
	SystemAnnouncer uniqueInstance
		on: MethodRepackaged do: [ :a | ann := a ].
	
	self addXYCategory. 
	firstPackage := self organizer  packageNamed: #XXXXX.
	secondPackage := self organizer  packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass:  class inCategory: '*yyyyy'.
	
	self assert: ann isNil.

	class organization classify: #stubMethod under: '*yyyyy-suncategory'.

	self assert: ann isNil.! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'GuillermoPolito 5/26/2012 23:09'!
testRecategorizeClassInUnknownCategoryRegisterTheClassInANewPackage
	"test that when we recategorize a class in category not yet registered in RPackage, a new rPackage is created with the class inside"
	
	|XPackage YPackage class|
	self addXCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self assert: (self organizer packageOf: class) = XPackage. 
	
	class category: 'YYYYY'.
	YPackage := self organizer packageNamed: #YYYYY.
	self assert: (self organizer packageOf: class) = YPackage. 
	self deny: (self organizer packageOf: class) = XPackage. ! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'GuillermoPolito 5/26/2012 23:10'!
testRecategorizeClassRegisterTheClassInTheNewPackage
	"test that when we recategorize a class, the new package in which it is defined include it"
	
	|XPackage YPackage class|
	self addXCategory.
	self addYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage := self organizer packageNamed: #YYYYY.
	
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self assert: (self organizer packageOf: class) = XPackage. 
	
	class category: 'YYYYY'.
	self assert: (self organizer packageOf: class) = YPackage. 
	self deny: (self organizer packageOf: class) = XPackage.		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'GuillermoPolito 5/26/2012 23:33'!
testRecategorizeClassRegisterTheClassMethodsInTheNewPackage

	"test that when we recategorize a class, the new package in which it is defined include all the methods defined in this class (not extensions)"
	
	| addedCount addedCat XPackage YPackage ZPackage class|
	self addXCategory.
	self addYCategory.
	Smalltalk organization addCategory: 'ZZZZZ'.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage := self organizer packageNamed: #YYYYY.
	ZPackage := self organizer packageNamed: #ZZZZZ.
	
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'method1' inClass: class inCategory: 'category'.
	self createMethodNamed: 'method2' inClass: class inCategory: 'category'.
	self createMethodNamed: 'method3' inClass: class inCategory: '*zzzzz'.
	self createMethodNamed: 'method4' inClass: class inCategory: '*yyyyy'.
	
	self assert: (XPackage includesDefinedSelector: #method1 ofClass: class ). 
	self assert: (XPackage includesDefinedSelector: #method2 ofClass: class ).
	self assert: (ZPackage includesExtensionSelector: #method3 ofClass: class ). 
	self assert: (YPackage includesExtensionSelector: #method4 ofClass: class ). 
	
	class category: 'YYYYY'.
	
	self assert: (YPackage includesDefinedSelector: #method1 ofClass: class ). 
	self deny: (XPackage includesDefinedSelector: #method1 ofClass: class ). 
	self deny: (XPackage includesExtensionSelector: #method1 ofClass: class ). 
	self assert: (YPackage includesDefinedSelector: #method2 ofClass: class ).
	self deny: (XPackage includesDefinedSelector: #method2 ofClass: class ). 
	self deny: (XPackage includesExtensionSelector: #method2 ofClass: class ). 
	self deny: (YPackage includesExtensionSelector: #method3 ofClass: class ). 
	self deny: (YPackage includesDefinedSelector: #method3 ofClass: class ). 
	self assert: (ZPackage includesExtensionSelector: #method3 ofClass: class ). 
	"I am not sure about this test. For now it does not pass, but I don't know what is the expecting behavior here."
	"self assert: (YPackage includesExtensionSelector: #method4 ofClass: class ). "! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'GuillermoPolito 5/26/2012 23:34'!
testRecategorizeClassUnregisterTheClassFromTheOldPackage
	"test that when we recategorize a class, the old package in which it was defined don't include it anymore"
	

	|XPackage YPackage class|
	self addXCategory.
	self addYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage := self organizer packageNamed: #YYYYY.
	
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self assert: (self organizer packageOf: class) = XPackage. 
	
	class category: 'YYYYY'.
	self deny: (XPackage includesClass: class). 		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'GuillermoPolito 5/26/2012 23:34'!
testRecategorizeClassUnregisterTheClassMethodsFromTheOldPackage
	"test that when we recategorize a class, the old package in which it was defined don't include its defined methods (not extensions) anymore"
	"self debug: #testRecategorizeClassUnregisterTheClassMethodsFromTheOldPackage"
		
	|XPackage YPackage ZPackage class|
	self addXCategory.
	self addYCategory.
	Smalltalk organization addCategory: 'ZZZZZ'.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage := self organizer packageNamed: #YYYYY.
	ZPackage := self organizer packageNamed: #ZZZZZ.
	
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'method1' inClass: class inCategory: 'category'.
	self createMethodNamed: 'method2' inClass: class inCategory: 'category'.
	self createMethodNamed: 'method3' inClass: class inCategory: '*zzzzz'.
	self createMethodNamed: 'method4' inClass: class inCategory: '*yyyyy'.
	
	self assert: (XPackage includesDefinedSelector: #method1 ofClass: class ). 
	self assert: (XPackage includesDefinedSelector: #method2 ofClass: class ).
	self assert: (ZPackage includesExtensionSelector: #method3 ofClass: class ). 
	self assert: (YPackage includesExtensionSelector: #method4 ofClass: class ). 
	
	class category: 'YYYYY'.
	self deny: (XPackage includesDefinedSelector: #method1 ofClass: class ). 
	self deny: (XPackage includesExtensionSelector: #method1 ofClass: class ). 
	self deny: (XPackage includesDefinedSelector: #method2 ofClass: class ). 
	self deny: (XPackage includesExtensionSelector: #method2 ofClass: class ). 
	self deny: (XPackage includesDefinedSelector: #method3 ofClass: class ). 
	self deny: (XPackage includesExtensionSelector: #method3 ofClass: class ). 
	self deny: (XPackage includesDefinedSelector: #method4 ofClass: class ). 
	self deny: (XPackage includesExtensionSelector: #method4 ofClass: class ).! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'GuillermoPolito 8/17/2012 17:34'!
testRecategorizeClassUpdateRaisesClassRecategorized
	"test that when we recategorize a class, the organizer is updated so that the class name point the the new RPackage"
	
	| XPackage YPackage class ann |
	
	ann := nil.
	SystemAnnouncer uniqueInstance
		on: ClassRepackaged do: [ :a |
			ann := a.
		].
	
	self addXCategory.
	self addYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	class category: 'YYYYY'.	
	
	self assert: ann notNil.
	self assert: ann classRepackaged equals: class.
	self assert: ann oldPackage equals: XPackage.
	self assert: ann newPackage equals: YPackage.! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'GuillermoPolito 5/26/2012 23:35'!
testRecategorizeClassUpdateTheOrganizer
	"test that when we recategorize a class, the organizer is updated so that the class name point the the new RPackage"
	
	|XPackage YPackage class|
	self addXCategory.
	self addYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	class category: 'YYYYY'.	
	self assert: (self organizer packageOf: class) = YPackage		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'GuillermoPolito 5/26/2012 23:36'!
testRecategorizeClassWithExtensionInTheExtendedPackage
	"test that when we recategorize a class with extension methods in the corresponding extended package, the extension methods become defined methods"

			|XPackage YPackage class|
			self addXCategory.
			self addYCategory.
			XPackage := self organizer packageNamed: #XXXXX.
			YPackage := self organizer packageNamed: #YYYYY.
			
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: #RPackageTestsExtensionMethod inClass: class  inCategory: '*yyyyy'.  
			self assert: (self organizer packageOf: class) = XPackage. 
			self assert: (YPackage extensionSelectors includes: #RPackageTestsExtensionMethod). 
			self deny: ((XPackage definedSelectorsForClass: class) includes: #RPackageTestsExtensionMethod). 
		
			class category: 'YYYYY'.
			self assert: (self organizer packageOf: class) = YPackage. 
			self deny: (self organizer packageOf: class) = XPackage. 
			self deny: ((XPackage definedSelectorsForClass: class) includes: #RPackageTestsExtensionMethod).
			self assert: ((YPackage definedSelectorsForClass: class)  includes: #RPackageTestsExtensionMethod). 
			self deny: (YPackage extensionSelectors includes: #RPackageTestsExtensionMethod). ! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'GuillermoPolito 5/26/2012 23:37'!
testRecategorizeClassWithMetaClassMethodsRegisterAllClassMethodsInTheNewPackage

	"test that when we recategorize a class (having methods defined in both instance and class side), the new package in which it is defined include all the methods (from instance and class side) defined in this class (not extensions)"
	
	| addedCount addedCat XPackage YPackage ZPackage class|
	self addXCategory.
	self addYCategory.
	Smalltalk organization addCategory: 'ZZZZZ'.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage := self organizer packageNamed: #YYYYY.
	ZPackage := self organizer packageNamed: #ZZZZZ.
	
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'method1' inClass: class inCategory: 'category'.
	self createMethodNamed: 'method2' inClass: class inCategory: 'category'.
	self createMethodNamed: 'method3' inClass: class inCategory: '*zzzzz'.
	self createMethodNamed: 'method4' inClass: class inCategory: '*yyyyy'.
	self createMethodNamed: 'method5' inClass: class theMetaClass inCategory: 'category'.
	self createMethodNamed: 'method6' inClass: class theMetaClass inCategory: 'category'.
	self createMethodNamed: 'method7' inClass: class theMetaClass inCategory: '*zzzzz'.
	
	self assert: (XPackage includesDefinedSelector: #method1 ofClass: class ). 
	self assert: (XPackage includesDefinedSelector: #method2 ofClass: class ).
	self assert: (ZPackage includesExtensionSelector: #method3 ofClass: class ). 
	self assert: (YPackage includesExtensionSelector: #method4 ofClass: class ).
	self assert: (XPackage includesDefinedSelector: #method5 ofMetaclassName: class name). 
	self assert: (XPackage includesDefinedSelector: #method6 ofMetaclassName: class name). 
	self assert: (ZPackage includesExtensionSelector: #method7 ofMetaclassName: class name ). 
	
	class category: 'YYYYY'.
	
	self assert: (YPackage includesDefinedSelector: #method1 ofClass: class ). 
	self deny: (XPackage includesDefinedSelector: #method1 ofClass: class ). 
	self deny: (XPackage includesExtensionSelector: #method1 ofClass: class ). 
	self assert: (YPackage includesDefinedSelector: #method2 ofClass: class ).
	self deny: (XPackage includesDefinedSelector: #method2 ofClass: class ). 
	self deny: (XPackage includesExtensionSelector: #method2 ofClass: class ). 
	self deny: (YPackage includesExtensionSelector: #method3 ofClass: class ). 
	self deny: (YPackage includesDefinedSelector: #method3 ofClass: class ). 
	self assert: (ZPackage includesExtensionSelector: #method3 ofClass: class ). 
	"lets check metaclass methods"
	self assert: (YPackage includesDefinedSelector: #method5 ofMetaclassName: class name). 
	self assert: (YPackage includesDefinedSelector: #method6 ofMetaclassName: class name). 
	self deny: (XPackage includesDefinedSelector: #method5 ofMetaclassName: class name). 
	self deny: (XPackage includesDefinedSelector: #method6 ofMetaclassName: class name). 
	self assert: (ZPackage includesExtensionSelector: #method7 ofMetaclassName: class name ). 
	"I am not sure about this test. For now it does not pass, but I don't know what is the expecting behavior here."
	"self assert: (YPackage includesExtensionSelector: #method4 ofClass: class ). "		! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 5/26/2012 23:37'!
testRemoveAllExtensionMethodsFromMetaAndInstanceSideUpdateOrganizerExtensionsDictionary
	"test that In a class (from the meta  and the instance side ), when we remove all the method extended by the same external package, the class get unregistered from the classExtendingPackagesMapping of the organizer "

	|XPackage  class YPackage|
	self addXYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage  := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'. 
	self createMethodNamed: 'stubMethod2' inClass: class theMetaClass  inCategory: '*yyyyy'. 
	
	self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
	self assert: (YPackage includesExtensionSelector: #stubMethod2 ofClass: class theMetaClass). 
	class removeSelector: #stubMethod.
	"there should be no differences made between class and metaClass:"
	self assert: ((self organizer extendingPackagesOf: class) includes: YPackage). 
	self assert: ((self organizer extendingPackagesOf: class theMetaClass) includes: YPackage). 
	class theMetaClass removeSelector: #stubMethod2.
	"there should be no differences made between class and metaClass:"
	self deny: ((self organizer extendingPackagesOf: class) includes: YPackage). 
	self deny: ((self organizer extendingPackagesOf: class theMetaClass) includes: YPackage). ! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 5/26/2012 23:38'!
testRemoveAllExtensionMethodsFromMetaClassUpdateOrganizerExtensionsDictionary
	"test that In a class (from the meta side), when we remove all the method extended by the same external package, the class get unregistered from the classExtendingPackagesMapping of the organizer "

	|XPackage  class YPackage|
	self addXYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage  := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass: class theMetaClass inCategory: '*yyyyy'. 
	self createMethodNamed: 'stubMethod2' inClass: class theMetaClass  inCategory: '*yyyyy'. 
	
	self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class theMetaClass). 
	self assert: (YPackage includesExtensionSelector: #stubMethod2 ofClass: class theMetaClass). 
	class theMetaClass removeSelector: #stubMethod.
	"there should be no differences made between class and metaClass:"
	self assert: ((self organizer extendingPackagesOf: class) includes: YPackage). 
	self assert: ((self organizer extendingPackagesOf: class theMetaClass) includes: YPackage). 
	class theMetaClass removeSelector: #stubMethod2.
	"there should be no differences made between class and metaClass:"
	self deny: ((self organizer extendingPackagesOf: class) includes: YPackage). 
	self deny: ((self organizer extendingPackagesOf: class theMetaClass) includes: YPackage). ! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 5/26/2012 23:38'!
testRemoveAllExtensionMethodsRemoveTheClassFromExtendedClasses
	"test that In a class, when we remove all the method extended by the same external package, the class get unregistered from the extended classes of the external package "

	|XPackage  class YPackage|
	self addXYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage  := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'. 
	self createMethodNamed: 'stubMethod2' inClass: class  inCategory: '*yyyyy'. 
	
	self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
	self assert: (YPackage includesExtensionSelector: #stubMethod2 ofClass: class). 
	class removeSelector: #stubMethod.
	self assert: (YPackage extendsClass: class). 
	class removeSelector: #stubMethod2.
	self deny: (YPackage extendsClass: class).! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 5/26/2012 23:38'!
testRemoveAllExtensionMethodsRemoveTheMetaClassFromExtendedClasses
	"test that In a class (from the class side), when we remove all the method extended by the same external package, the class get unregistered from the extended classes of the external package "

	|XPackage  class YPackage|
	self addXYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage  := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass: class theMetaClass inCategory: '*yyyyy'. 
	self createMethodNamed: 'stubMethod2' inClass: class theMetaClass  inCategory: '*yyyyy'. 
	
	self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class theMetaClass ). 
	self deny: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
	self assert: (YPackage includesExtensionSelector: #stubMethod2 ofClass: class theMetaClass ).
	self deny: (YPackage includesExtensionSelector: #stubMethod2 ofClass: class ). 
	class theMetaClass removeSelector: #stubMethod.
	self assert: (YPackage extendsClass: class theMetaClass ). 
	self assert: (YPackage extendsClass: class ). 
	class theMetaClass removeSelector: #stubMethod2.
	self deny: (YPackage extendsClass: class).
	self deny: (YPackage extendsClass: class theMetaClass).! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 5/26/2012 23:39'!
testRemoveAllExtensionMethodsUpdateOrganizerExtensionsDictionary
	"test that In a class, when we remove all the method extended by the same external package, the class get unregistered from the classExtendingPackagesMapping of the organizer "

	|XPackage  class YPackage|
	self addXYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage  := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'. 
	self createMethodNamed: 'stubMethod2' inClass: class  inCategory: '*yyyyy'. 
	
	self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
	self assert: (YPackage includesExtensionSelector: #stubMethod2 ofClass: class). 
	class removeSelector: #stubMethod.
	self assert: ((self organizer extendingPackagesOf: class) includes: YPackage). 
	class removeSelector: #stubMethod2.
	self deny: ((self organizer extendingPackagesOf: class) includes: YPackage).! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on categories' stamp: 'GuillermoPolito 5/26/2012 23:39'!
testRemoveCategoryRemoveThePackage
	"test that when we remove a category, the corresponding RPackage is removed from the organizer"
	
	|XPackage class|
	self addXCategory.
	self assert: (self organizer includesPackageNamed: #XXXXX).
	Smalltalk organization removeCategory: 'XXXXX'.
	self deny: (self organizer includesPackageNamed: #XXXXX).! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on categories' stamp: 'GuillermoPolito 5/26/2012 23:40'!
testRemoveCategoryWhenMCPackageExistDoesNotRemoveThePackage
	"test that when we remove a category (usually from the class browser), and that a corresponding MCPackage exist, the RPackage still exist till the MCPackage has been unloaded"
	
	|XPackage class mCPackage|
	"let's create the category and the MCPackage"
	self addXCategory.
	mCPackage := MCWorkingCopy forPackage: (MCPackage new name: 'XXXXX').
	"we make sure that the RPackage has been create (only one time)"
	self assert: (self organizer includesPackageNamed: #XXXXX).
	self assert: (self organizer packages select: [:each | each name = 'XXXXX']) size = 1.
	"we remove the category"
	Smalltalk organization removeCategory: 'XXXXX'.
	"we make sure the RPackage still exist"
	self assert: (self organizer includesPackageNamed: #XXXXX).
	
	"for now, this no longer the expected behavior. See comment in RPackageOrganizer>>update:"
	"mCPackage unregister.
	self deny: (anOrganizer includesPackageNamed: #XXXXX)."	! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'GuillermoPolito 5/26/2012 23:40'!
testRemoveClassUnregisterTheClassDefinedMethodsFromItsPackage
	"test that when we remove a class, the class methods defined in the parent package (not extensions) are all removed  from its parent Package"
	
			|XPackage  class|
			self addXCategory.
			XPackage := self organizer packageNamed: #XXXXX.
			class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
			self createMethodNamed: 'stubMethod' inClass: class  inCategory: 'classic category'. 
			
			self assert: (XPackage includesDefinedSelector: #stubMethod ofClass: class). 
			Smalltalk removeClassNamed: 'NewClass'. 
			self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class). 
			self deny: (XPackage includesExtensionSelector: #stubMethod ofClass: class). 
			self deny: (XPackage includesSelector: #stubMethod ofClass: class). ! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'GuillermoPolito 5/26/2012 23:40'!
testRemoveClassUnregisterTheClassExtensionMethodsFromTheCorrespondingPackage
	"test that when we remove a class, the class extension methods are all removed  from theire corresponding parent Package"
	
	|XPackage YPackage class|
	self addXYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'. 
	
	self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
	Smalltalk removeClassNamed: 'NewClass'. 
	self deny: (YPackage includesDefinedSelector: #stubMethod ofClass: class). 
	self deny: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
	self deny: (YPackage includesSelector: #stubMethod ofClass: class). ! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'GuillermoPolito 5/26/2012 23:40'!
testRemoveClassUnregisterTheClassFromItsPackage
	"test that when we remove a class, the class is removed from its parent Package"	

	|XPackage  class|
	self addXCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self assert: (XPackage includesClass: class). 
	Smalltalk removeClassNamed: 'NewClass'. 
	self deny: (XPackage includesClass: class)! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'GuillermoPolito 5/26/2012 23:41'!
testRemoveClassUpdateTheorganizer
	"test that when we remove a class, the organizer is updated so that the class is no longer present in the  'classPackageDictionary' dictionary"
	
	|XPackage  class|
	self addXCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self assert: (self organizer includesPackageBackPointerForClass: class). 
	Smalltalk removeClassNamed: 'NewClass'. 
	self deny: (self organizer includesPackageBackPointerForClass: class)! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 5/26/2012 23:41'!
testRemoveExtensionMethodDoesNotRemoveExtendingPackage
	"test that when we remove a method that is an extension from an external package, the package is not removed from the the class extending packages if another extension still exist"

	|XPackage  class YPackage|
	self addXYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage  := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'. 
	self createMethodNamed: 'stubMethod2' inClass: class  inCategory: '*yyyyy'. 
	
	self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
	class removeSelector: #stubMethod.
	self assert: ((self organizer extendingPackagesOf: class) includes: YPackage).  ! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 5/26/2012 23:42'!
testRemoveExtensionMethodDoesRemoveExtendingPackage
	"test that when we remove a method that is an extension from an external package, the package is  removed from the the class extending packages if no other extension method exist"

	|XPackage  class YPackage|
	self addXYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage  := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'. 
	
	self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
	class removeSelector: #stubMethod.
	self assert: ((self organizer extendingPackagesOf: class) isEmpty).  ! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 5/26/2012 23:42'!
testRemoveExtensionMethodRemoveMethodFromItsPackage
	"test that when we remove a method that is an extension from an external package, the method is removed from this package"

	|XPackage  class YPackage|
	self addXYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage  := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'. 
	
	self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
	class removeSelector: #stubMethod.
	self deny: (YPackage includesDefinedSelector: #stubMethod ofClass: class). 
	self deny: (YPackage includesExtensionSelector: #stubMethod ofClass: class). 
	self deny: (YPackage includesSelector: #stubMethod ofClass: class).! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on MCPackages' stamp: 'StephaneDucasse 6/2/2012 19:42'!
testRemoveMCPackageRemoveThePackage
	"test that when we remove a MC Package, the corresponding RPackage is removed from the organizer"
	
	|XPackage class|
	MCWorkingCopy forPackage: (MCPackage new name: 'XXXXX').
	self assert: (self organizer includesPackageNamed: #XXXXX).
	(self allManagers detect: [:each | each packageName = 'XXXXX']) unregister.
	"for now, this is no longer the expected behavior. See comment in RPackageOrganizer>>update:"
	"self deny: (anOrganizer includesPackageNamed: #XXXXX)."
	! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 5/26/2012 23:44'!
testRemoveMethodComingFromTraitDoesNotRemoveMethodFromTraitPackage
	"test that when we remove a class method coming from a trait, the method is not removed from the parent package of the trait"
	
	|XPackage YPackage class trait|
	self addXCategory.
	self addYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	trait := self createNewTraitNamed: 'NewTrait' inCategory: 'YYYYY'.
	self createMethodNamed: 'stubMethod' inClass: trait  inCategory: 'classic protocol'. 
	class setTraitComposition: (trait) asTraitComposition.
	
	self assert: (YPackage includesDefinedSelector: #stubMethod ofClass: trait). 
	self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class). 
	class removeSelector: #stubMethod.
	self assert: (YPackage includesDefinedSelector: #stubMethod ofClass: trait). 
	self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class). ! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 5/26/2012 23:44'!
testRemoveMethodRemoveMethodFromItsPackage
	"test that when we remove a method, the method is remod from the package in which the method is defined"
	
	|XPackage  class|
	self addXCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass: class  inCategory: 'classic category'. 
	
	self assert: (XPackage includesDefinedSelector: #stubMethod ofClass: class). 
	class removeSelector: #stubMethod.
	self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class). 
	self deny: (XPackage includesExtensionSelector: #stubMethod ofClass: class). 
	self deny: (XPackage includesSelector: #stubMethod ofClass: class). ! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 5/26/2012 23:44'!
testRemoveTraitMethod
	"test that when we remove a method from a trait used by a class, the method is removed from the trait package normally"
	
	|XPackage YPackage class trait|
	self addXCategory.
	self addYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	trait := self createNewTraitNamed: 'NewTrait' inCategory: 'YYYYY'.
	self createMethodNamed: 'stubMethod' inClass: trait  inCategory: 'classic protocol'. 
	class setTraitComposition: (trait) asTraitComposition.
	
	self assert: (YPackage includesDefinedSelector: #stubMethod ofClass: trait). 
	self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class). 
	trait removeSelector: #stubMethod.
	self deny: (YPackage includesDefinedSelector: #stubMethod ofClass: trait). 
	self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class). ! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on categories' stamp: 'GuillermoPolito 5/26/2012 23:45'!
testRenameCategoryAlsoRenameAllExtensionProtocols
	"test that when we rename a category, all corresponding extension protocols in the system are renamed"
	"self debug: #testRenameCategoryAlsoRenameAllExtensionProtocols"
	
	|XPackage YPackage ZPackage classInY classInZ| 
	self addXCategory.
	self addYCategory.
	self addZCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage := self organizer packageNamed: #YYYYY.
	ZPackage := self organizer packageNamed: #ZZZZZ.
	
	classInY := self createNewClassNamed: 'ClassInYPackage' inCategory: 'YYYYY'.
	classInZ := self createNewClassNamed: 'ClassInZPackage' inCategory: 'ZZZZZ'. 
	
	self createMethodNamed:  'extensionFromXInClassInY'  inClass: classInY   inCategory: '*XXXXX'. 
	self createMethodNamed:  'longNameExtensionFromXInClassInY'  inClass: classInY   inCategory: '*XXXXX-subcategory'.     
	self createMethodNamed:  'extensionFromXInClassInZ'  inClass: classInZ   inCategory: '*XXXXX'.     
	    
	Smalltalk organization renameCategory: 'XXXXX' toBe: 'NewCategoryName'.
	
	self assert: (XPackage name = 'NewCategoryName').
	self assert: (classInY >> #extensionFromXInClassInY) category = '*NewCategoryName'.
	self assert: (classInY >> #longNameExtensionFromXInClassInY) category = '*NewCategoryName'.
	self assert: (classInZ >> #extensionFromXInClassInZ) category = '*NewCategoryName'.! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on categories' stamp: 'GuillermoPolito 5/26/2012 23:45'!
testRenameCategoryChangeTheNameOfThePackage
	"test that when we rename a category, the RPackage corresponding is updated with this new name"
	
	|XPackage| 
	self addXCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	
	Smalltalk organization renameCategory: 'XXXXX' toBe: 'YYYYY'.
	self assert: (XPackage name = 'YYYYY').! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on categories' stamp: 'GuillermoPolito 5/26/2012 23:45'!
testRenameCategoryUpdateTheOrganizer
	"test that when we rename a category, the organizer dictionary is update with this new name, so that we can access the package with this new name as key"

	|XPackage| 
	self addXCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	Smalltalk organization renameCategory: 'XXXXX' toBe: 'YYYYY'.
	self assert: (self organizer packageNamed: 'YYYYY' asSymbol) = XPackage.
	self deny: (self organizer includesPackageNamed: #XXXXX).! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'GuillermoPolito 5/26/2012 23:45'!
testRenameClassUpdateClassDefinedInTheParentPackage
	"test that when we rename a class, the classes Defined in the parent package includes the new name"
	"self debug: #testRenameClassUpdateClassDefinedInTheParentPackage"
	
	|XPackage  class refactoring|
	self addXCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	
	class := self createNewClassNamed: 'RPackageOldStubClass' inCategory: 'XXXXX'.	
	class rename: 'RPackageNewStubClass'.
	
	self assert: (XPackage includesClassNamed: 'RPackageNewStubClass'). 
	self deny: (XPackage includesClassNamed: 'RPackageOldStubClass').! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'GuillermoPolito 5/26/2012 23:46'!
testRenameClassUpdateClassDefinedSelectorsInTheParentPackage
	"test that when we rename a class, the  'classDefinedSelectors' dictionary of the parent package is updated with the new name. There fore we test that we can correctly access the selector from the package by specifying the right name (the new name)"
	
	|XPackage  class refactoring|
	self addXCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	class := self createNewClassNamed: 'RPackageOldStubClass' asSymbol inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass: class  inCategory: 'classic category'.
	
	class rename: 'RPackageNewStubClass'.
	
	self assert: (XPackage includesDefinedSelector: #stubMethod ofClassName: 'RPackageNewStubClass' asSymbol ). 
	self deny: (XPackage includesDefinedSelector: #stubMethod ofClassName: 'RPackageOldStubClass' asSymbol ).! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'GuillermoPolito 5/26/2012 23:46'!
testRenameClassUpdateClassExtensionSelectorsInTheExtendingPackages
	"test that when we rename a class, the  'classExtensionSelectors' dictionaries of the extending packages  are updated with the new name. Therfore we test that we can correctly access the selectors from the package by specifying the right name (the new name)"

	|XPackage YPackage class refactoring|
	self addXYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'RPackageOldStubClass' asSymbol inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'.
	
	class rename: 'RPackageNewStubClass'.
	
	self assert: (YPackage includesExtensionSelector: #stubMethod ofClassName: 'RPackageNewStubClass' asSymbol ). 
	self deny: (YPackage includesExtensionSelector: #stubMethod ofClassName: 'RPackageOldStubClass' asSymbol ).! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'GuillermoPolito 5/26/2012 23:46'!
testRenameClassUpdateMetaclassDefinedSelectorsInTheParentPackage
	"test that when we rename a class, the  'metaclassDefinedSelectors' dictionary of the parent package is updated with the new name. Ther fore we test that we can correctly access the selector from the package by specifying the right name (the new name)"
	
	|XPackage  class refactoring|
	self addXYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	class := self createNewClassNamed: 'RPackageOldStubClass' asSymbol inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClassSideOfClass: class  inCategory: 'classic category'.
	
	class rename: 'RPackageNewStubClass'.
	
	self assert: (XPackage  includesDefinedSelector: #stubMethod ofMetaclassName: 'RPackageNewStubClass' asSymbol ). 
	self deny: (XPackage  includesDefinedSelector: #stubMethod ofMetaclassName: 'RPackageOldStubClass' asSymbol ).! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'GuillermoPolito 5/26/2012 23:47'!
testRenameClassUpdateMetaclassExtensionSelectorsInTheExtendingPackages
	"test that when we rename a class, the  'metaclassExtensionSelectors' dictionaries of the extending packages  are updated with the new name. Ther fore we test that we can correctly access the selectors from the package by specifying the right name (the new name)"
	
	|XPackage YPackage class refactoring|
	self addXYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'RPackageOldStubClass' asSymbol inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClassSideOfClass: class  inCategory: '*yyyyy'.
	
	class rename: 'RPackageNewStubClass'.
	
	self assert: (YPackage includesExtensionSelector: #stubMethod ofMetaclassName: 'RPackageNewStubClass' asSymbol ). 
	self deny: (YPackage includesExtensionSelector: #stubMethod ofMetaclassName: 'RPackageOldStubClass' asSymbol ).! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'GuillermoPolito 5/26/2012 23:47'!
testRenameClassUpdateOrganizerClassExtendingPackagesMapping
	"test that when we rename a class, the classExtendingPackages dictionary of the organizer is updated with the new name, so that we can access the packages when specifying the new name"
	
	|XPackage YPackage class refactoring|
	self addXYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'RPackageOldStubClass' inCategory: 'XXXXX'.
	self createMethodNamed: #stubMethod inClass: class inCategory: '*yyyyy'. 
	
	class rename: 'RPackageNewStubClass'.
	
	self assert: ((self organizer extendingPackagesOfClassNamed: 'RPackageNewStubClass' asSymbol) includes: YPackage). 
	self deny: ((self organizer extendingPackagesOfClassNamed: 'RPackageOldStubClass' asSymbol) includes: YPackage).! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on classes' stamp: 'GuillermoPolito 5/26/2012 23:47'!
testRenameClassUpdateOrganizerClassPackageMapping
	"test that when we rename a class, the classPackage dictionary of the organizer is updated with the new name, so that we can access the package when specifying the new name"
	
	|XPackage class refactoring|
	self addXYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	class := self createNewClassNamed: 'RPackageOldStubClass' inCategory: 'XXXXX'.
	
	class rename: 'RPackageNewStubClass'.
	
	self assert: ((self organizer packageOfClassNamed: 'RPackageNewStubClass' asSymbol) = XPackage). 
	self assert: ((self organizer packageOfClassNamed: 'RPackageOldStubClass' asSymbol) = nil).! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on categories' stamp: 'EstebanLorenzano 8/3/2012 15:29'!
testRenameUnknownCategoryCreatesNewRPackage
	"test that when we rename a category that is not registered in RPackage , it does not raise errors and simply create a new package. We need this behaviour as for now, create a new category with the class browser does not emit the corrects events, and therefore RPackage can not be directly updated"
	
	SystemAnnouncer uniqueInstance suspendAllWhile: [
		self addXCategory.
		].
	self deny: (self organizer includesPackageNamed: #XXXXX).
	Smalltalk organization renameCategory: 'XXXXX' toBe: 'YYYYY'.
	self assert: (self organizer includesPackageNamed: #YYYYY).! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on protocols' stamp: 'GuillermoPolito 5/26/2012 23:48'!
testReorganizeClassByAddingNewProtocolDoesNothing
	"test that when we reoganized a class by adding a category, nothing change from  RPackage point of vue."
	
	|XPackage YPackage class| 
	self addXCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'newMethod' inClass: class inCategory: 'xxxxx'.  
	class organization addCategory: 'accessing' before: nil.
	
	self assert: (XPackage includesClass: class).
	self assert: (XPackage includesDefinedSelector: #newMethod ofClass: class).
	self assert: (XPackage extensionMethods size = 0).! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on protocols' stamp: 'EstebanLorenzano 7/27/2012 20:19'!
testReorganizeClassByRemovingClassicCategoryRemoveMethodsFromTheParentPackage
	"test that when we reoganized a class by removing a classic category (a category not beginning with '*'), the methods from this category are removed from the parent RPackage of the class"
	
	|XPackage YPackage class| 
		
	self addXCategory.
	XPackage := self organizer packageNamed: 'XXXXX'.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass: class  inCategory: 'classic category'.
	class removeCategory: 'classic category'.
	self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class).! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on protocols' stamp: 'GuillermoPolito 5/26/2012 23:49'!
testReorganizeClassByRemovingExtensionCategoryRemoveMethodsFromTheExtendingPackage
	"test that when we reoganized a class by removing an extension category (a category beginning with '*'), all the methods that were inside are removed from the extending package"
	 
	|XPackage YPackage class| 
	self addXYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage  := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	
	self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'.
	
	class removeCategory: '*yyyyy'.
	
	self deny: (YPackage includesExtensionSelector: #stubMethod ofClass: class).
	self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class).! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on protocols' stamp: 'GuillermoPolito 5/26/2012 23:49'!
testReorganizeClassByRenamingClassicCategoryToExtensionCategoryMoveAllMethodsFromParentPackageToExtendingPackage
	"self debug:#testReoganizeClassByRenamingClassicCategoryToExtensionCategoryMoveAllMethodsFromParentPackageToExtendingPackage"
	"test that when we reoganized a class by renaming a  classic category (a category not beginning with '*') to an extension category, all the methods are moved from the  parent package of the class to the extending package"
	|XPackage YPackage class| 
	self addXYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage  := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	
	self createMethodNamed: 'stubMethod' inClass: class  inCategory: 'classic category'.
	self createMethodNamed: 'stubMethod2' inClass: class  inCategory: 'classic category'.
	
	class organization renameCategory: 'classic category' toBe: '*yyyyy'.
	
	self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class).
	self assert: (YPackage includesExtensionSelector: #stubMethod2 ofClass: class).
	self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class).
	self deny: (XPackage includesDefinedSelector: #stubMethod2 ofClass: class).! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on protocols' stamp: 'GuillermoPolito 5/26/2012 23:49'!
testReorganizeClassByRenamingClassicCategoryToExtensionCategoryMoveMethodsFromParentPackageToExtendingPackage
	"test that when we reoganized a class by renaming a  classic category (a category not beginning with '*') to an extension category, all the methods are moved from the  parent package of the class to the extending package"

	|XPackage YPackage class| 
	self addXYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage  := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass: class  inCategory: 'classic category'.
	class organization renameCategory: 'classic category' toBe: '*yyyyy'.
	self assert: (YPackage includesExtensionSelector: #stubMethod ofClass: class).
	self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class)! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on protocols' stamp: 'GuillermoPolito 5/26/2012 23:49'!
testReorganizeClassByRenamingExtensionCategoryToAnotherExtensionCategoryAddMethodsToTheNewPackage
	"test that when we reoganized a class by renaming an extension category (a category beginning with '*') to another extension category, all the methods are moved to the new extendingPackage"
	
	|XPackage YPackage ZPackage class| 
	self addXYZCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage  := self organizer packageNamed: #YYYYY.
	ZPackage  := self organizer packageNamed: #ZZZZZ.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'.
	class organization renameCategory: '*yyyyy' toBe: '*zzzzz'.
	self assert: (ZPackage includesExtensionSelector: #stubMethod ofClass: class).
	self deny: (YPackage includesExtensionSelector: #stubMethod ofClass: class).
	self deny: (XPackage includesDefinedSelector: #stubMethod ofClass: class)! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on protocols' stamp: 'GuillermoPolito 5/26/2012 23:50'!
testReorganizeClassByRenamingExtensionCategoryToClassicCategoryMoveAllMethodsFromExtendingPackageToParentPackage
	"test that when we reoganized a class by renaming an extension category (a category beginning with '*') to a classic category, all the methods are moved from the  extendingPackage to the parent package of the class"
	"self debug: #testReoganizeClassByRenamingExtensionCategoryToClassicCategoryMoveAllMethodsFromExtendingPackageToParentPackage"
	
	|XPackage YPackage class| 
	self addXYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage  := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	
	self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'.
	self createMethodNamed: 'stubMethod2' inClass: class  inCategory: '*yyyyy'.
	
	class organization renameCategory: '*yyyyy' toBe: 'classic category'.
	
	self deny: (YPackage includesExtensionSelector: #stubMethod ofClass: class).
	self deny: (YPackage includesExtensionSelector: #stubMethod2 ofClass: class).
	self assert: (XPackage includesDefinedSelector: #stubMethod ofClass: class).
	self assert: (XPackage includesDefinedSelector: #stubMethod2 ofClass: class).! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on protocols' stamp: 'GuillermoPolito 5/26/2012 23:50'!
testReorganizeClassByRenamingExtensionCategoryToClassicCategoryMoveMethodsFromExtendingPackageToParentPackage
	"test that when we reoganized a class by renaming an extension category (a category beginning with '*') to a classic category, all the methods are moved from the  extendingPackage to the parent package of the class"
	
	|XPackage YPackage class| 
	self addXYCategory.
	XPackage := self organizer packageNamed: #XXXXX.
	YPackage  := self organizer packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass: class  inCategory: '*yyyyy'.
	class organization renameCategory: '*yyyyy' toBe: 'classic category'.
	self deny: (YPackage includesExtensionSelector: #stubMethod ofClass: class).
	self assert: (XPackage includesDefinedSelector: #stubMethod ofClass: class)! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 8/17/2012 17:38'!
testRepackagedAnnouncementWhenModifyMethodByMovingFromClassicCategoryToExtensionCategory
	
	| ann class firstPackage secondPackage|
	
	ann := nil.
	SystemAnnouncer uniqueInstance
		on: MethodRepackaged do: [ :a | ann := a ].
	
	self addXYCategory. 
	firstPackage := self organizer  packageNamed: #XXXXX.
	secondPackage := self organizer  packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass:  class inCategory: 'classic category'.
	
	class organization classify: #stubMethod under: '*yyyyy'.		

	self assert: ann notNil.
	self assert: ann methodRepackaged selector equals: #stubMethod.
	self assert: ann oldPackage equals: firstPackage.
	self assert: ann newPackage equals: secondPackage.
	ann := nil.

	class organization classify: #stubMethod under: '*yyyyy-suncategory'.

	self assert: ann isNil.! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 8/17/2012 17:43'!
testRepackagedAnnouncementWhenModifyMethodByMovingFromDifferentExtensionCategories

	| ann class firstPackage secondPackage thirdPackage |
	
	ann := nil.
	SystemAnnouncer uniqueInstance
		on: MethodRepackaged do: [ :a | ann := a ].
	
	self addXYZCategory. 
	firstPackage := self organizer  packageNamed: #XXXXX.
	secondPackage := self organizer packageNamed: #YYYYY.
	thirdPackage := self organizer packageNamed: #ZZZZZ.
	
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass:  class inCategory: '*yyyyy'.
	
	class organization classify: #stubMethod under: '*zzzzz'.

	self assert: ann notNil.
	self assert: ann methodRepackaged selector equals: #stubMethod.
	self assert: ann oldPackage equals: secondPackage.
	self assert: ann newPackage equals: thirdPackage.! !

!RPackageMCSynchronisationTest methodsFor: 'tests - operations on methods' stamp: 'GuillermoPolito 8/17/2012 17:40'!
testRepackagedAnnouncementWhenModifyMethodByMovingFromExtensionCategoryToClassicCategory

	| ann class firstPackage secondPackage|
	
	ann := nil.
	SystemAnnouncer uniqueInstance
		on: MethodRepackaged do: [ :a | ann := a ].
	
	self addXYCategory. 
	firstPackage := self organizer  packageNamed: #XXXXX.
	secondPackage := self organizer  packageNamed: #YYYYY.
	class := self createNewClassNamed: 'NewClass' inCategory: 'XXXXX'.
	self createMethodNamed: 'stubMethod' inClass:  class inCategory: '*yyyyy'.
	
	class organization classify: #stubMethod under: 'classic one'.		

	self assert: ann notNil.
	self assert: ann methodRepackaged selector equals: #stubMethod.
	self assert: ann oldPackage equals: secondPackage.
	self assert: ann newPackage equals: firstPackage.! !

!RPackageMCSynchronisationTest methodsFor: 'tests' stamp: 'GuillermoPolito 5/26/2012 23:50'!
testisDefinedAsPackageOrSubPackageInMC
	"test whether a given category is defined as a MC package or whether it is defined as top part of package"
	
	"self debug: #testisDefinedAsSubPackageInMC"
	"self debug: #testisDefinedAsPackageOrSubPackageInMC"
	
	self assert: (self organizer isDefinedAsPackageOrSubPackageInMC: #'Compression').
	self deny: (self organizer isDefinedAsPackageOrSubPackageInMC: #'Compression-Archives').
	"Indeed compression-archives is a category but not a MC package"
	
	self assert: (self organizer isDefinedAsPackageOrSubPackageInMC: #'Collections').
	"#Collection is not defined as a package but as a package #'Collections-Unordered'"
	self assert: (self organizer isDefinedAsPackageOrSubPackageInMC: #'Collections-Unordered').
	self deny: (self organizer isDefinedAsPackageOrSubPackageInMC: #'Collections-Unordered-Test').! !

Announcer subclass: #SystemAnnouncer
	instanceVariableNames: 'suspended private'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Announcements'!

SystemAnnouncer class
	instanceVariableNames: 'announcer'!

!SystemAnnouncer commentStamp: '<historical>' prior: 0!
This class plays the role of the announcer for events raised by the Pharo system.

BEWARE: You should not need to subclass me. Think about just using me to send your announces (if you need system behavior) or using your own announcers as instance or class  variables.!

SystemAnnouncer class
	instanceVariableNames: 'announcer'!

!SystemAnnouncer methodsFor: '*RPackage-SystemIntegration' stamp: 'GuillermoPolito 8/17/2012 17:24'!
classRepackaged: aClass from: aPackage to: anotherPackage
	self announce: (ClassRepackaged
						classRepackaged: aClass
						oldPackage: aPackage 
						newPackage: anotherPackage)! !

!SystemAnnouncer methodsFor: '*RPackage-SystemIntegration' stamp: 'GuillermoPolito 8/17/2012 17:38'!
methodRepackaged: aMethod from: aPackage to: anotherPackage
	self announce: (MethodRepackaged
						methodRepackaged: aMethod
						oldPackage: aPackage 
						newPackage: anotherPackage)! !

Object subclass: #MCPackage
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Monticello-Base'!

!MCPackage methodsFor: '*RPackage-SystemIntegration' stamp: 'BenjaminVanRyseghem 4/14/2011 15:57'!
correspondingRPackage

	| symbol |
	symbol := Symbol readFromString: '#', self name.
	^ RPackageOrganizer default packageNamed: symbol ifAbsent: [ nil ]! !

Object subclass: #PackageChecker
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage-SystemIntegration'!

!PackageChecker methodsFor: 'check' stamp: 'StephaneDucasse 7/30/2010 14:03'!
check
	"self new check"
	
	"only work for top level package overlapping.
	Foo and Foo-Core, but not Foo-Core and Foo-Core-Test"
	| packageNames dict overlappingPackages |
	overlappingPackages := OrderedCollection new.
	packageNames := MCWorkingCopy allManagers collect: [:each | each packageName].
	dict := packageNames groupedBy: [:each | each copyUpTo: $-].
	dict copy keysAndValuesDo: [:k :v | v size = 1 ifTrue: [ dict removeKey: k]].
	dict keysAndValuesDo: [:k :v | (v includes: k) 
									ifTrue: [
										overlappingPackages add: v.
										Warning signal: 'Overlapping packages: ', v printString]].
	^ overlappingPackages ! !

Object subclass: #RPackage
	instanceVariableNames: 'classDefinedSelectors metaclassDefinedSelectors classExtensionSelectors metaclassExtensionSelectors classes name classTags'
	classVariableNames: 'PackageGlobalOrganizer'
	poolDictionaries: ''
	category: 'RPackage-Core'!

!RPackage commentStamp: 'StephaneDucasse 7/25/2011 21:55' prior: 0!
A RPackage is a new implementation of package. Note that it does not touch classes but it is just a layer on top of classes therefore removing a method from a package does not change the underlying class. 


Instance Variables
	classDefinedSelectors:		Dictionary
	classExtensionSelectors:		Dictionary
	metaclassDefinedSelectors:		Dictionary
	metaclassExtensionSelectors:		Dictionary
	packageName:		String
	classes: OrderedCollection of Symbols
	classTags: is a mapping from tags to classNames

The reason we use four dictionaries is that this way we only store #Point in both as the class and the metaclass are involved. We do not have to with Point_class problem when comparing defined and extended classes. The idea is taken from the RBSelectorEnvironment. 					
		
Class Invariant one:
	Classes should not contain metaclass name, but only class names
	
	

Implementation notes 
====================
This class went over 3 internal representations implementation. 
	- first: 	a list of class + two dictionaries: class * methods
	This was not good since we add to go all the time over the list of classes.
	- second: 4 dictionaries class * selectors 
	This was not good since we want to have defined classes without methods. 
	- third: 4 dictionaries + definedClasses

Originally I wanted to avoid to have a defined class list and I wanted to avoid to have to declare the class as defined. But this is not really good since
	- we will want to know if a class definition (without method for example is defined in a given package)
	- second this is easier to hook the packageOf: behavior (we register the package class when the class is 	added to the package).

defined classes information is redundant with the dictionary keys of defined methods but we would have to check and register the class to the packageOrganizer the first time a method is defined then also check on remove to unregister the class from the packageOrganizer.

Adding a method does not define the class as a defined package class. This has to be done explictly. The reason for this choice is that a class should register to the packageOrganizer and that I do not want to do it each time a method is added and I do not want to test it each time a method is added. Now this is done only when the class is declared as defined. 
We could also give the complete freedom to the client to register the class but I thought it was a good compromise. 
	
	
ClassTag are tags that can be associated to classes. They help user organizing their class internal. 
So that we can have a package infrastructure as follows:
	Package1
		ClassA
		ClassB
	in case there is no tags associated to the package
	or
	Package2	
		Tag1
			ClassA
			ClassB
		Tag2
			ClassC
			ClassD
			ClassE
			ClassA		
	
Todo
====
	Next: 
	- finish 
	- build up a synchronizer that import PackageInfo.
	

	

	
!

!RPackage methodsFor: '*RPackage-SystemIntegration' stamp: 'StephaneDucasse 4/13/2011 22:45'!
importCategory
	"import all the classes of the class categories matching the receiver name (following package info semantics)." 
	
	| cat classesInCat |
	cat := Smalltalk globals organization categories select: [:each | each beginsWith:  self name ]. 
	classesInCat := cat gather: [:c | Smalltalk globals organization listAtCategoryNamed: c].
	classesInCat do: [:each | 
						self importClass: (Smalltalk globals at: each).
						self importClass: (Smalltalk globals at: each) class]! !

!RPackage methodsFor: '*RPackage-SystemIntegration' stamp: 'StephaneDucasse 7/29/2010 21:14'!
importCategory: aCat forClass: aClass 
	"import all the methods of a category as defined in the receiver." 
	
		(aClass organization listAtCategoryNamed: aCat)
			do: [:each | self addMethod: (aClass >> each)]
	
	! !

!RPackage methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrilledelaunay 5/31/2011 15:56'!
importClass: aClass 
	"import a class already created but not attached to a package to the receiver. 
	Handle also *- convention. Methods defined in *category are not added to the package.
	Pay attention that it will not import anything from the metaClass side
	"
	
	| protocols |
	self addClassDefinition: aClass.
	protocols := aClass organization categories 
		reject: [:each | (each first = $*) and: [ ( '*', self name asLowercase match: each asLowercase) not] ].
		"we reject all extensions except the ones that have the same name as the package - highly improbable but who knows"
	protocols do: [:each | self importProtocol: each forClass: aClass]
	
	! !

!RPackage methodsFor: '*RPackage-SystemIntegration' stamp: 'StephaneDucasse 7/29/2010 21:16'!
importProtocol: aProtocol forClass: aClass 
	"import all the methods of a protocol as defined in the receiver." 
	
		(aClass organization listAtCategoryNamed: aProtocol)
			do: [:each | self addMethod: (aClass >> each)]
	
	! !

Object subclass: #RPackageOrganizer
	instanceVariableNames: 'classPackageMapping packages classExtendingPackagesMapping debuggingName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'RPackage-Core'!

RPackageOrganizer class
	instanceVariableNames: 'default'!

!RPackageOrganizer commentStamp: '<historical>' prior: 0!
A RPackageOrganizer is responsible for providing all the package currently defined in the system.
In addition it provides a back pointer from the class to its package.

The classPackageMapping and the classExtendingPackageMapping should be moved in the future to the classes themselves. 


For tests or actions that could destroy the package organizer,  do not access directly the singleton of RPackageOrganizer.
Use instead 
		RPackage withOrganizer: aNewOrganizer do: ablock
			or via RPackage organizer
		
	
RPackageOrganizer fillUp will fill up the system from the current PackageOrganizer
	"self fillUp"
	
-----------------------------------------------------------------------------------------------------------------------------------------------	
	
	
A rpackageOrganizer update itself when some changes are made in the system. It does that by registering to a systemAnnoucer, specifying an action when an annoucement is triggered.
Here is what I (Cyrille Delaunay) propose to do for each annocuement triggered:

SystemCategoryAddedAnnouncement 	
	=> I would just register a new RPackage (if it does not already exist) in the RPackageOrganizer
	
SystemCategoryRemovedAnnouncement 
     => I would just unregister the RPackage concerned from the organizer

SystemCategoryRenamedAnnouncement
     => I would update the RPackage concerned, by changing its name
     => I would update the 'packages' dictionary of the organizer, putting the new name as key

SystemClassAddedAnnouncement 
    => Import the class in the RPackage concerned (RPackage >> importClass:)
    => Register the class in the 'classPackageMapping' dictionary of the organizer (RPackageOrganizer >> registerPackage:forClass)
    (=> maybe we should pay attention if both the class and the metaclass launch this kind of event ?)

SystemClassRecategorizedAnnouncement
    => I would update the old RPackage concerned:
            => unregister the class
            => unregister all defined methods of the class
    => I would update the new RPackage:
            => Import the class in the RPackage (importClass:)
    => I would update the organizer:
            => update the 'classPackageDictionary' to point on the new RPackage

 
SystemClassRemovedAnnouncement
    => I would update the RPackake concerned
             => unregister the class
             => unregister all defined methods of the class
    => I would update the organizer:
             => update the 'classPackageDictionary' to remove the class

SystemClassRenamedAnnouncement
    => I would update the RPackage in which the class is defined:
             => update the 'classDefinedSelectors' dictionary (replace the old key by the new one)
             => update the 'metaclassDefinedSelectors' dictionary (replace the old key by the new one)
    => I would update all RPackages extending this class
             => update the 'classExtensionsSelectors' dictionary (replace the old key by the new one)
             => update the 'metaclassclassExtensionsSelectors' dictionary (replace the old key by the new one)
    => I would update the organizer
             => update the 'classPackageDictionary' to replace the key with the new class name
             => update the 'classExtendingPackagesMapping' to replace the key with the new class name
                          
SystemClassReorganizedAnnouncement 
    (=> I guess we should check if extensions have not been added or removed ? 
      (to retrieve this information, the only thing I found is ClassDescription >> organization, and then check each category begining with '*' and compare with the organizer. seems to be painful, no?))
	=> when an extension is removed, all methods inside are removed. Therefore, the MethodRemovedAnnounecement will do the job. Not sur this one still usefull

SystemProtocolAddedAnnouncement
    => I don't see anything to do for this annoucement

SystemProtocolRemovedAnnoucement
    => If the category is an extension from a package, I would move all the methods concerned, from the extending RPackage to the class RPackage

SystemMethodAddedAnnouncement
       => I would check the category in which the method has been defined
               => if it correspond to an extending package -> add the method to the extending RPackage
               => if not, add the method to the class parentPackage

SystemMethodModifiedAnnouncement
       this annoucement can correspond to several kind of modifications:
	       *  a method has been renamed
                       => I would update the rPackage in which the method is defined to replace the old selector by the new one
		* a method has been move to another category 
			-maybe from a classic category to an extending package
                             => we should move the method from the  method class parentPackage to extendingPackage package
			-maybe from an extending package to another extending package
                             => we should move the method from the  extendingPackage package to the other extendingPackage package
			-maybe from an extending package to a classic category
                             =>  we should move the method from the  extendingPackage to the method class parentPackage
		        -maybe from a classic category to another classic category
                             => we have nothing to do
			

SystemMethodRecategorizedAnnouncement
          same thing than above

SystemMethodRemovedAnnouncement
       => I would simply remove the method from the RPackage in which it is register!

RPackageOrganizer class
	instanceVariableNames: 'default'!

!RPackageOrganizer class methodsFor: '*RPackage-SystemIntegration' stamp: 'StephaneDucasse 6/3/2012 22:36'!
isPackageDefinedForClass: aClass

	^ (self default packageOf: aClass) notNil! !

!RPackageOrganizer class methodsFor: '*RPackage-SystemIntegration' stamp: 'GuillermoPolito 8/3/2012 14:20'!
methodAdded: anEvent
	"precondition: package exist, class exist"
	
	| methodCategory |
	methodCategory := anEvent protocol.
	(methodCategory isEmptyOrNil or:[ methodCategory first ~= $* ]) 
		ifFalse: [
			(self isPackageDefinedForClass: anEvent methodClass)
					ifFalse: [self packageClass new named: (Smalltalk organization categoryOfElement: anEvent methodClass theNonMetaClass asSymbol) ].
		].! !

!RPackageOrganizer class methodsFor: '*RPackage-SystemIntegration' stamp: 'sd 4/23/2011 23:31'!
registerInterestToSystemAnnouncement

	self default unregisterInterestToSystemAnnouncement.
	"To make sure that we do not have it twice registered"
	self default registerInterestToSystemAnnouncement! !

!RPackageOrganizer class methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrille.delaunay 11/26/2010 10:23'!
unregisterInterestToSystemAnnouncement

	self default unregisterInterestToSystemAnnouncement! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'EstebanLorenzano 8/3/2012 14:22'!
announcer 
	^SystemAnnouncer uniqueInstance private! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'GuillermoPolito 8/5/2012 13:34'!
fullyRemoveClassNamed: className
	"Remove the class, the class backpointer, the extensions and the extension backPointer from the receiver and the class involved with the class named: className. className is a class name and should not be a metaclass one. "	

	| rPackage |	
	rPackage := self packageOfClassNamed: className.
	rPackage ifNil: [^self].
	rPackage removeClassDefinitionNamed: className.
	rPackage removeAllMethodsFromClassNamed: className. 
	(self extendingPackagesOfClassNamed: className)
		do: [:each | each removeAllMethodsFromClassNamed: className. ].
	
	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'GuillermoPolito 6/3/2012 01:35'!
hasPackageExactlyMatchingExtensionName: anExtensionName
	^packages keys anySatisfy: [ :aSymbol | (aSymbol sameAs: anExtensionName asSymbol)]! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'GuillermoPolito 6/3/2012 02:24'!
hasPackageForProtocol: aProtocolName inClass: aClass
	"According aProtocolName is an extension protocol, will look for a matching package. If no matching package is found ,  return nil. If aProtocolName is not an extension protocol, return the parent package of aClass"
	
	|tmpProtocol|
	tmpProtocol := aProtocolName ifNil: [''].
	^ (tmpProtocol beginsWith: '*') not or: [ self hasPackageMatchingExtensionName:  (tmpProtocol copyWithout: $*) ]! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'GuillermoPolito 6/3/2012 01:36'!
hasPackageMatchingExtensionName: anExtensionName

	|tmpPackage tmpPackageName|
	
	(self hasPackageExactlyMatchingExtensionName: anExtensionName)
		ifTrue: [ ^true ].
	
	^packages keys anySatisfy: [:aSymbol | aSymbol asString,'*'  match: anExtensionName ].
	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'StephaneDucasse 4/13/2011 20:18'!
packageExactlyMatchingExtensionName: anExtensionName
	"only look for a package for which the name match 'anExtensionName', making no difference about case. Return nil if no package is found"
	
	|tmpPackage tmpPackageName|
	packages keys do: [:aSymbol |
		(aSymbol sameAs: anExtensionName asSymbol)
			ifTrue: [ tmpPackage := self packageNamed: aSymbol ]].
	^ tmpPackage! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrilledelaunay 5/13/2011 13:46'!
packageForProtocol: aProtocolName inClass: aClass
	"According aProtocolName is an extension protocol, will look for a matching package. If no matching package is found ,  return nil. If aProtocolName is not an extension protocol, return the parent package of aClass"
	
	|tmpProtocol|
	tmpProtocol := aProtocolName ifNil: [''].
	^ (tmpProtocol beginsWith: '*')
		ifTrue: [  (self packageMatchingExtensionName:  (tmpProtocol copyWithout: $*)) ]
		ifFalse: [ aClass package ]! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrille.delaunay 11/25/2010 13:40'!
packageMatchingExtensionName: anExtensionName

	"return nil if no package i found"
	|tmpPackage tmpPackageName|
	"we first look if their is a package matching exactly the name specified"
	packages keys do: [:aSymbol |
		(aSymbol sameAs: anExtensionName asSymbol)
			ifTrue: [^ self packageNamed: aSymbol]
		].
	
	"if no package was found, we try to find one matching the begining of the name specified"
	tmpPackageName := ''.
	packages keys do: [:aSymbol |
		(aSymbol asString,'*'  match: anExtensionName)
			ifTrue: [
				"we keep the longest package name found"
				(aSymbol size > tmpPackageName size) ifTrue: [
					tmpPackageName := aSymbol.
					tmpPackage := self packageNamed: aSymbol
					].
				]
		].
	^ tmpPackage! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'EstebanLorenzano 8/3/2012 14:09'!
registerInterestToAnnouncer: anAnnouncer	
	"There should be only one"
	anAnnouncer unsubscribe: self.
	
	" Announcer initialize"
	self flag: #todo. "Fiinish monticello integration (commented below)"
	
	anAnnouncer weak
		on: CategoryAdded send: #systemCategoryAddedActionFrom: to: self;
		on: CategoryRemoved send: #systemCategoryRemovedActionFrom: to: self;
		on: CategoryRenamed send: #systemCategoryRenamedActionFrom: to: self;
		on:  ClassAdded send: #systemClassAddedActionFrom: to: self;
		on:  ClassCommented do: [:ann | 
				 "Only re-emit public corresponding announcement"
			"Not yet implemented?"
			];
		on:  ClassRecategorized send: #systemClassRecategorizedActionFrom: to: self;
		on:  ClassRemoved send: #systemClassRemovedActionFrom: to: self;
		on:  ClassRenamed send: #systemClassRenamedActionFrom: to: self;
		on:  ClassReorganized send: #systemClassReorganizedActionFrom: to: self;
		on:  MethodAdded send: #systemMethodAddedActionFrom: to: self;
		on:  MethodModified send: #systemMethodModifiedActionFrom: to: self;
		on:  MethodRecategorized send: #systemMethodRecategorizedActionFrom: to: self;
		on:  MethodRemoved send: #systemMethodRemovedActionFrom: to: self;
		on:  ProtocolAdded send: #systemProtocolAddedActionFrom: to: self;
		on:  ProtocolRemoved send: #systemProtocolRemovedActionFrom: to: self;
		"Monticello announcements (old protocol)"
"		on: MCVersionCreated send: #update: to: self;
		on: MCPackageModified send: #update: to: self;"
		on: MCWorkingCopyCreated send: #updateAfterNewMCPackageRegistred: to: self;
		on: MCWorkingCopyDeleted send: #updateAfterNewMCPackageUnregistred: to: self	
		! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'GuillermoPolito 7/31/2012 11:59'!
registerInterestToSystemAnnouncement
	"self registerInterestToSystemAnnouncement"
	<systemEventRegistration>
	
	self registerInterestToAnnouncer: self announcer! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'EstebanLorenzano 8/7/2012 11:24'!
systemCategoryAddedActionFrom: ann

	| rPackage packageSymbol |
	self flag: #fixMe. "a categoryName should always be a symbol, but for some reason it is keeping a string some times" 
	packageSymbol := ann categoryName asSymbol.
	(self includesPackageNamed: packageSymbol)
		ifFalse: [ 
			rPackage := self packageClass named: packageSymbol.
			self registerPackage: rPackage ]! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'EstebanLorenzano 8/7/2012 11:10'!
systemCategoryRemovedActionFrom: ann

	| rPackage categoryNameSymbol |
	categoryNameSymbol := ann categoryName asSymbol.
	"here we check that a corresponding MCPackage does not still exist. In this case we should keep the RPackage"
	self allManagers detect: [:each | self category: each packageName matches: categoryNameSymbol asString] ifNone: [
		rPackage := packages at: categoryNameSymbol ifAbsent: [^ self].
		"if no MCPackage exist, we unregister the package from the RPackageOrganizer"
		self basicUnregisterPackageNamed: categoryNameSymbol.	
		].
	
! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'GuillermoPolito 8/3/2012 14:39'!
systemCategoryRenamedActionFrom: ann

	| rPackage classesAndProtocolsToRename|
	"we update the rpackage itself with the new name"
	rPackage := self packageNamed: ann oldCategoryName asSymbol ifAbsent: [self packageClass new].
	rPackage name: ann newCategoryName.
	
	"we update the organizer"
	packages at: ann newCategoryName asSymbol put: rPackage.
	packages removeKey: ann oldCategoryName asSymbol ifAbsent: [self reportBogusBehaviorOf:   #systemCategoryRenamedActionFrom: ].
	self flag: #cyrille. "Should we not update the backPointer mapping too?"
	"CD: What do you mean by backpointerMapping? By telling a RPackage to rename itself, it should update itself all its internal. 
	Then it just left all references in the organizer to update. For now I see the 'packages' variable. "
	
	"we also rename all the extension protocols in the system with the new name"
	classesAndProtocolsToRename := (rPackage extensionMethods collect: [:aCompiledMethod |
		aCompiledMethod methodClass -> aCompiledMethod category
		]) asSet.
	classesAndProtocolsToRename do: [:anAssociation |
		anAssociation  key organization silentlyRenameCategory:  anAssociation  value toBe: '*', ann newCategoryName
		].
		
! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'GuillermoPolito 8/3/2012 14:25'!
systemClassAddedActionFrom: ann

	| class rPackage categoryNameSymbol |
	class := ann classAffected.
	categoryNameSymbol := class category.
	rPackage := packages at: categoryNameSymbol ifAbsent: [
		"a class may be defined for a package that does not yet exist. In this case we have to create first the new package"
		self registerPackage: (self packageClass named: categoryNameSymbol).
		].		
	"when we add the class, the parent category should normally already be register correctly at the RPackageOrganizer"
	"this next line should import the class and all its methods: "
	rPackage importClass: class.
	rPackage importClass: class theMetaClass
	"maybe the last update should be (is?) automatically done when adding or removing from a RPackage."
	"self default registerPackage: rPackage forClass: class"
	"maybe have to check for class and metaClass that will maybe both throw an event?
	=> seems to be correctly handled by RPackage"! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'GuillermoPolito 8/17/2012 17:35'!
systemClassRecategorizedActionFrom: ann

	| class newRPackage oldRPackage newCategoryName oldCategoryName|
	"self flag: 'seems to be ok'."
	class := ann classAffected.	
	newCategoryName := ann newCategory asSymbol.
	oldCategoryName := ann oldCategory asSymbol.
	oldRPackage := self packageNamed: oldCategoryName.
	newRPackage := self packageNamed: newCategoryName ifAbsent: [
		"temporary code. For now adding a new category in the system does not emit correctly the corresponding event. The organizer is therefore not able to update itself. So here, if we don't find a matching package, we create a new one"
		self registerPackage: (self packageClass named: newCategoryName)
		].
	"we have to update the old package"
	oldRPackage removeClass: class.	
	"we have to update the new package"
	newRPackage importClass: class.
	"we should also import the metaClass"
	newRPackage importClass: class theMetaClass	.

	SystemAnnouncer uniqueInstance classRepackaged: class from: oldRPackage to: newRPackage.
	
	"we have to update the RPackageOrganizer.
	maybe the last update should be automatically done when adding or removing from a RPackage
	"
	"self default unregisterPackage: oldRPackage forClass: class.
	self default registerPackage: newRPackage forClass: class"! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'GuillermoPolito 8/5/2012 13:27'!
systemClassRemovedActionFrom: ann
	"A class has been removed, we should update the package adequately."
	
	| class className rPackage categoryName |	
	class := ann classRemoved.
	className  := class originalName.
		
	self fullyRemoveClassNamed: className.
	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'GuillermoPolito 8/3/2012 14:27'!
systemClassRenamedActionFrom: ann
	"When a class is renamed, we update its package as well as the organizer"
	
	| newName oldName class rPackage extendingRPackages packageName |
	class := ann classAffected.
	packageName := ann category.
	oldName := ann oldName.
	newName := ann newName.
	rPackage := self packageNamed: packageName.
	extendingRPackages := self extendingPackagesOfClassNamed: oldName.
	
	rPackage updateDefinedClassNamed: oldName withNewName: newName.
	"we have to update all RPackages extending this class"
	extendingRPackages do: [:aRPackage | 
		aRPackage updateExtensionForClassNamed: oldName withNewName: newName].
		
	"we have to update the RPackageOrganizer.
	update the 'classPackageDictionary' to replace the key with the new class name"
	self unregisterPackage: rPackage forClassName: oldName.
	self registerPackage: rPackage forClassName: newName.	
		
	"update the 'classExtendingPackagesMapping' to replace the key with the new class name"
	extendingRPackages do: [:aRPackage |
		self unregisterExtendingPackage: aRPackage forClassName: oldName.
		self registerExtendingPackage:  aRPackage forClassName: newName   
		].
	self flag: #cyrille. "we will see that."
	"maybe we should remove also the key from the extendedPackages dictionary"
	"CD: This is what are doing the lines above I think"

	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrilledelaunay 5/10/2011 13:46'!
systemClassReorganizedActionFrom: ann
	"when a class is reorganized, we have to check if an extension has not been added"
	
	|class|
	self flag: 'Not sure this method is usefull. If we remove a category, it seems that we will remove the methods inside, therefore the remove method announcement will do the job. Then all other modifications does not impact the packages'.
	"CD: All tests pass with this version"
	"The event ReorganizedEvent is lauched for several cases:
		- we rename a protocol (see 'renameCategory:toBe: '). if The category is not empty, SystemMethodRecategorizedAnnouncement will also be emitted // THE WORK WILL BE DONE BY  SystemMethodRecategorizedAnnouncement
		- we sort (modify the order of) protocols (see 'sortCategories') // WE DON'T CARE HERE
		- we add a protocol (see 'addCategory:before:') // WE DON'T CARE HERE'
		- we remove a protocol (see 'removeCategory:' 'removeEmptyCategories') // THE WORK SHOULD BE DONE BY  SystemMethodRemovedAnnouncement
	".
! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'GuillermoPolito 8/3/2012 14:18'!
systemClassReorganizedActionFromBis: ann
	"only to keep old code somewhere"
	"when a class is reorganized, we have to check if an extension has not been added"
	
	|class|
	self flag: 'Not sure this method is usefull. If we remove a category, it seems that we will remove the methods inside, therefore the remove method announcement will do the job. Then all other modifications does not impact the packages'.
	"The event ReorganizedEvent is lauched for several cases:
		- we rename a protocol (see 'renameCategory:toBe: '). if The category is not empty, SystemMethodRecategorizedAnnouncement will also be emitted // THE WORK WILL BE DONE BY  SystemMethodRecategorizedAnnouncement
		- we sort (modify the order of) protocols (see 'sortCategories') // WE DON'T CARE HERE
		- we add a protocol (see 'addCategory:before:') // WE DON'T CARE HERE'
		- we remove a protocol (see 'removeCategory:' 'removeEmptyCategories') // THE WORK SHOULD BE DONE BY  SystemMethodRemovedAnnouncement
		
		
	So the work here is quite big :) The event only provide a reference to the class that has been modified. So I guess we have to:
		=> we check that each extending rPackage contain all methods from the corresponding extension category (as extension methods)
		=> the parentPackage of the class includes all methods defined in classic categories
	".
	class := ann methodClass.
	class organization categories do: [:aString |
		(aString beginsWith: '*')
			ifTrue: [
				"here we deal with extension-categories"
				|extendingPackageName extendingPackage|
				extendingPackageName := aString copyWithout: $*.
				extendingPackage := self packageMatchingExtensionName: extendingPackageName.
				extendingPackage ifNotNil: [
					"we check that all extension method for this classs in the rpackage correspond to all methods in the extension category"
					(class methodsInCategory: aString) do: [:aMethodsSelector |
					
					"If the method origin is not the one of the event, we do not care about that method"
					(class >> aMethodsSelector) origin = ann methodClass ifFalse: [^self].
						((extendingPackage extensionMethodsForClass: class) includes: (class methodDict at: aMethodsSelector))
							ifFalse: [
								"if a method has not been yet registered in the rPackage, we add it"
								((class methodDict at: aMethodsSelector) packageFromOrganizer: self ) removeMethod: (class methodDict at: aMethodsSelector).
								extendingPackage addMethod: (class methodDict at: aMethodsSelector).
								]  
						]
					]
				]
			ifFalse: [
				"here we deal with classic categories"
				(class methodsInCategory: aString) do: [:aMethodSelector |
					"aMethodSelector can come from a trait used by class. Such a method should always belong to to the trait and therefore to the parent package of the trait"
					(class traitOrClassOfSelector: aMethodSelector) = class ifFalse: [^self].
					((class package definedMethodsForClass: class) includes: (class methodDict at: aMethodSelector ) )
						ifFalse: [
							((class methodDict at: aMethodSelector) packageFromOrganizer: self) removeMethod: (class methodDict at: aMethodSelector).
							class package addMethod: (class methodDict at: aMethodSelector)
							]
					]
				]
			
		].
	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'GuillermoPolito 8/3/2012 14:18'!
systemMethodAddedActionFrom: ann

	|class selector protocol method rPackage |

	class := ann methodClass.
	protocol := ann protocol ifNil: [ '' ].
	selector := ann selector.
	method := ann method.
	
	"If the method origin is not the one of the event, we do not care about that method"
	method origin = ann methodClass ifFalse: [^ self].
	
	"we have to register the method in the parent RPackage of the class. 
	to do that, we first have to look if the method is an extension from an external package:"
	"1) in some cases, it can happen that protocol is nil (I do not know yet when). That's why we first check if protocol is nil. If it is, we just add the method worrying about anything. 
	This case can happen for exemple when running MCMethodDefinitionTest >> #testLoadAndUnload. Comment the nil check and should get an error."
	"This case come from CompiledMethod >> packageFromOrganizer:"
	
	rPackage := (self hasPackageForProtocol: protocol inClass: method methodClass) 
			ifTrue: [ self packageForProtocol: protocol inClass: method methodClass ]
			ifFalse: [ self registerPackage: (self packageClass named: (protocol copyWithout: $*)) ].
	rPackage addMethod: method.
	
	
	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'GuillermoPolito 8/3/2012 14:18'!
systemMethodModifiedActionFrom: ann

	"this annoucement can correspond to several kind of modifications:
		=> a method has been renamed
		=> a method has been move to another category // SHOULD BE TREATED WITH SystemMethodRecategorizedEvent
		(
			-maybe from the classic category to an extending package
			-maybe from an extending package to another extending package
			-maybe from an extending package to a classic category
			- maybe from a classic category to another classic category
			)
	"
	| oldMethod newMethod methodPackage |
	newMethod := ann newMethod.
	oldMethod := ann oldMethod.
	self flag: #cyrille. 
	"This is not clear that we need to do soemthing here since the method is removed and added so the other events should be handled correctly
	We should remove this method to see if this is working."
	
	"If the method origin is not the one of the event, we do not care about that method"
	newMethod origin = ann methodClass ifFalse: [^ self].

	methodPackage := newMethod packageFromOrganizer: self.
	"maybe the name of the method has changed"
	oldMethod selector = newMethod selector
		ifFalse: [ methodPackage updateSelector: oldMethod selector inClass: oldMethod methodClass withNewSelector: newMethod selector].
		

	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'GuillermoPolito 8/17/2012 17:35'!
systemMethodRecategorizedActionFrom: ann
	"a protocol has been renamed
	a method has been removed. // WILL BE TREATED WITH SystemMethodRemovedAnnouncement
	a method has been moved to another category 
		-maybe from the classic category to an extending package
		-maybe from an extending package to another extending package
		-maybe from an extending package to a classic category
		- maybe from a classic category to another classic category"
		
	|oldProtocol newProtocol method methodPackage destinationPackage|
	self flag: #cyrille.
	oldProtocol := ann oldProtocol.
	"old protocol might be nil:"
	oldProtocol asString ifNil: [oldProtocol := ''].
	newProtocol := ann  newProtocol.
	method := ann methodRecategorized.
	
	"1). See compiledMethod >> packageFromOrganizer:"
	
	"we have to pay attention that this announcement can come from TraitDescription>>updateOrganizationSelector:oldCategory:newCategory: . In this context, it can happen that the method referenced by the event is nil. In this case I think we should change nothing"
	
	
	method isNil ifFalse: [
		"If the method origin is not the one of the event, we do not care about that method"
		method origin = ann methodClass ifFalse: [^ self].
		
		methodPackage := method packageFromOrganizer: self.
		"if newProtocol is nil, it means that we are in a 'methodRemoved' case. Nothing should be modified (all the work will be done when catching methodRemovedAnnouncement) " 
		newProtocol isNil ifTrue: [^self].
		"it can happen that oldProtocol is Nil. in this case we should do nothing"
		oldProtocol ifNil: [^ self].
		"maybe the category has changed"
		newProtocol asLowercase = oldProtocol asLowercase
			ifFalse: [
				"according the extension specified is valid or not, the method will be defined as an extension of an external package or in the class parent package"
				destinationPackage := (self hasPackageForProtocol: newProtocol inClass: method methodClass)
					ifTrue: [ self packageForProtocol: newProtocol inClass: method methodClass ]
					ifFalse: [ method methodClass package ].
				methodPackage := (self hasPackageForProtocol: oldProtocol inClass: method methodClass)
					ifTrue: [ self packageForProtocol: oldProtocol inClass: method methodClass ]
					ifFalse: [ method methodClass package ].

				"then, we will remove the method from the method package, and add it to the destination package (we check before that the source package and the destination package are different)."
				methodPackage = destinationPackage
					ifFalse: [
						methodPackage removeMethod: method.
						destinationPackage addMethod: method.
						
						SystemAnnouncer uniqueInstance
											methodRepackaged: method
											from: methodPackage
											to: destinationPackage.
						]
				]
		]
	
	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'GuillermoPolito 8/5/2012 13:28'!
systemMethodRemovedActionFrom: ann

	| method methodPackage |
	method := ann methodAffected.
	"If the method origin is not the one of the event, we do not care about that method"
	method origin = ann methodClass ifFalse: [^ self].

	ann methodClass isMeta
		ifFalse: [ 
			methodPackage := self packageDefiningOrExtendingSelector: ann selector inClassNamed: ann methodClass theNonMetaClass originalName.
			methodPackage removeSelector: ann selector ofClassName: ann methodClass theNonMetaClass originalName.
		]
		ifTrue: [ 
			methodPackage := self packageDefiningOrExtendingSelector: ann selector inMetaclassNamed: ann methodClass theNonMetaClass originalName.
			methodPackage removeSelector: ann selector ofMetaclassName: ann methodClass theNonMetaClass originalName.
		]! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrille.delaunay 11/18/2010 11:42'!
systemProtocolAddedActionFrom: ann

	"for now I don't see anything to do'"! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'cyrilledelaunay 5/13/2011 13:53'!
systemProtocolRemovedActionFrom: ann
	"to be unregistered"
	
	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'StephaneDucasse 4/13/2011 21:16'!
systemTraitDefinitionModifiedActionFrom: ann
	"this kind of anounecement will be used when:
		=> we create a new trait.
		=> we declare new traits to use in a trait"
	|trait|
	trait := ann itemTrait.
	! !

!RPackageOrganizer methodsFor: '*RPackage-SystemIntegration' stamp: 'EstebanLorenzano 8/3/2012 14:09'!
unregisterInterestToSystemAnnouncement
	"self unregisterInterestToSystemAnnouncement"

	SystemAnnouncer uniqueInstance unsubscribe: self! !
